/*
  ChessPrn program by Ed Maillet
  @(#)drivers.c	1.0 Dated 3/93
  Copyright (C) 1993  Ed Maillet
  Modified and based on Notation 4.7 by Henry Thomas
 */

#ifdef __STDC__
#include <stdlib.h>
#endif
#include<stdio.h>
#include <string.h>
#include <ctype.h>

#include "config.h"
#include "chesstyp.h"
#include "notation.h"
#include "movegen.h"
#include "movedo.h"
#include "drivers.h"
#ifdef __STDC__
static int roque_to_move(depl *m);
static void init_buffer(format *d, int side);
static void flush_buffer(format *d);
static void output_move_generic(format *dr, depl *d);
static void output_variation_generic (format *dr, int inout);
static void output_text_generic(format *dr, int type, char *string, int code);
static void output_init_ascii(format *dr);
static void output_board_ps(format *dr,game *g);
static void output_init_roff(format *dr);
static void output_init_tex(format *dr);
static void output_text_tex(format *dr, int type, char * string, int code);
static void output_board_tex(format *dr,game *g);
static void output_end_tex(format *dr);
static void output_init_gnu(format *dr);
static void output_endgame_xchess(format *dr);
static void output_init_zarkov(format *dr);
static void output_end_zarkov(format *dr);
static void output_endgame_zarkov(format *dr);
static char CMMapPiece(int piece);
static char CMconfig(int flag);
static char CMTurnCnt(int flag);
static void CMconvert(char *buffer,unsigned char *lobyte,unsigned char *hibyte);
static void output_init_cm(format *dr);
static void output_end_cm(format *dr);
static void output_endgame_cm(format *dr);
static void output_endgame_pgn(format *dr);
static void FlushPGNStrings(int type);
static void output_endgame_chessarc(format *dr);
static void output_text_wps(format *dr, int type, char *string, int code);
static void output_init_wps(format *dr);
static void output_end_wps(format *dr);
static void output_board_wps(format *dr,game *g);
static void null_driver(void);
#endif

#define CHECK 3
#define RESET 2
#define SET 1
/* current column of print (moves, variations, and short comments (!!) ) */
static int curCol=0;

/* postscript characters translation table
   one entry per piece
   each entry has four fields:
   - white piece on white case
   - white piece on black case
   - black piece on ...
   ..
 */
#define PSINDEX(a,b) ((((a)==WHITE)?0:2)+(((b)%2)?0:1))

static char * postscript_table[][4] = {
  { " ", "x",     " ", "x"     }, /* void */
  { "k", "\\373", "K", "\\360" }, /* king */
  { "q", "\\317", "Q", "\\316" }, /* queen */
  { "r", "\\250", "R", "\\345" }, /* rook */
  { "b", "\\272", "B", "\\365" }, /* bishop */
  { "n", "\\265", "N", "\\366" }, /* knight */
  { "p", "\\271", "P", "\\270" }  /* pawn */
};

/* TeX table for using the chess figure in board design */
static char * texboard_table[][4] = {
  /*   W/W      W/B      B/W      B/B */
  { " ", "*", " ", "*" }, /* void */
  { "K", "K", "k", "k" }, /* king */
  { "Q", "Q", "q", "q" }, /* queen */
  { "R", "R", "r", "r" }, /* rook */
  { "B", "B", "b", "b" }, /* bishop */
  { "N", "N", "n", "n" }, /* knight */
  { "P", "P", "p", "p" }  /* pawn */
};

/* chars used for ascii board print */
static char ascboard[] = ASCBOARD_STRING ;

/* TeX table for using the chess figures in move description */
/* P.T. macros render this table useless as english symbols
   are active char in analysis mode
   */
static char * latex_table[] = {
  "", /* null */
/*  "{\\Fig K}", "{\\Fig Q}", "{\\Fig R}", "{\\Fig B}", "{\\Fig N}",  "" */
  "K", "Q", "R", "B", "N",  ""
    /* last entry = pawn; not represented, otherwise "{\\Fig P}" */
};

/* various tex symbols */
static char FigDash[] = "\\FigDash";
static char FigCapt[] = "*"; /* "*" is an active char with P.T. macros */
static char FigDots[] = ":"; /* ":" is an active char for "..." */
static char FigDot[] = "\\FigDot";

#define G_ROQUE  "O-O-O"
#define P_ROQUE  "O-O"

/* variation symbols */
static char varsymb[][2] = { { '{', '}' }, {'{','}'} };

static char * com_tex[] = {
#define CHESSSYMB(LET,LASC,SASC,TEX,PS,ENG,FRA) TEX,
#include "chesssym.def"
 ""
 };
#undef CHESSSYMB

static char * com_ps[] = {
#define CHESSSYMB(LET,LASC,SASC,TEX,PS,ENG,FRA) PS,
#include "chesssym.def"
 ""
 };
#undef CHESSSYMB

static FILE * ftmp ;
/* PGN strings - used by PGN, CM and chessarc drivers */

static char EventString[80];
static char SiteString[80];
static char DateString[15];
static char RoundString[10];
static char WhiteString[60];
static char BlackString[60];
static char ResultString[15];
static char CommentString[80];

/* ---------------- output functions ---------------- */

/* convert a roque in term of king's move */
#ifdef __STDC__
static int roque_to_move(depl *m)
#else
static int roque_to_move(m)
     depl * m;
#endif
{

  m->piece = KING;
  m->fromcol   = 5;
  if (m->type == GRANDROQUE)
    m->tocol = 3;
  else
    m->tocol = 7;

  if (m->whiteturn)
    m->fromlig = m->tolig = 1;
  else
    m->fromlig = m->tolig = 8;

  return(TRUE);
}

/* (kind of) buffering of output */
#ifdef __STDC__
static void init_buffer(format *d, int side)
#else
static void init_buffer(d,side)
     format * d;
     int side ;
#endif
{
  switch (d->type) {
  case D_ASCII:
    if (side != BLACK) (void) sprintf(d->white_buffer,"...");
    if (side != WHITE) (void) sprintf(d->black_buffer,"   ");
    break;
  case D_PS:
    if (side != BLACK) (void) sprintf(d->white_buffer," B (...) show ");
    if (side != WHITE) (void) sprintf(d->black_buffer," B (   ) show ");
    break;
  case D_TEX:
    if (side != BLACK) (void) sprintf(d->white_buffer,"%s",FigDots);
    if (side != WHITE) (void) sprintf(d->black_buffer,"~");
    break;
  default:
    if (side != BLACK) d->white_buffer[0] = '\0' ;
    if (side != WHITE) d->black_buffer[0] = '\0' ;
    break;
  }
}

/* this procedure is responsible for PRINTING the move */
#ifdef __STDC__
static void flush_buffer(format *d)
#else
static void flush_buffer(d)
     format * d;
#endif
{
  char outFormat[20];
  char *newLine;
  char *tabChar;

  int lineLen;
  /* if we have been interupted (by a comment, a board display etc...
     if the move is black
     we display <movenumber> ... <blackmove>
     */

  if(d->type==D_PS)
    { newLine=" N\n"; tabChar=" T "; }
  else
    { newLine="\n"; tabChar="\t"; }

  lineLen=d->type==D_PS?140:80;
  if(d->type!=D_CHESSARC)
  {
  if(!(d->type==D_GNU || d->type==D_XCHESS) && !d->column)
    {
     if(((curCol+15)>=lineLen) || d->newline)
       {
        if(!d->newline) (void) fprintf(d->outfile,"%s",newLine);
        curCol=0;
        d->newline=FALSE;
       }
    }
  else if(d->iswhiteturn) { (void) fprintf(d->outfile,"%s",newLine); }
        else if(d->intrupt==FALSE) {(void) fprintf(d->outfile,"%s",tabChar);}
  } /* !D_CHESSARC */

  if ((d->intrupt == TRUE) && (d->iswhiteturn == FALSE))
     {
      switch (d->type)
        {
         case D_PS:
              (void) fprintf(d->outfile,"( ) show %s B ( ... ) show %s \n",
                             d->move_buffer,d->black_buffer);
              curCol +=strlen(d->move_buffer)+strlen(d->black_buffer);
              break;
         case D_TEX:
              (void) fprintf(d->outfile,
	 	     " %s\\ %s\\ %s",
		     d->move_buffer,FigDots,d->black_buffer);
              break;
         case D_CHESSARC:
              (void) fprintf(d->outfile," %-6s\n",d->black_buffer);
              break;
         case D_GNU:
         case D_XCHESS:
             /* no special case for GNU */
             (void) fprintf(d->outfile," %s",d->black_buffer);
             break;
         case D_ZARKOV:
             (void) fprintf(d->outfile," %s",d->black_buffer);
             break;
         case D_CM:
             { unsigned char hibyte,lobyte;
               CMconvert(d->black_buffer,&lobyte,&hibyte);
               (void) fprintf(d->outfile,"%c%c%c%c",hibyte,lobyte,0x0,0x0);
               CMTurnCnt(BLACK);
             }
             break;
         default:
             strcpy(outFormat,d->column?"%3s.%9s\t%9s":" %s. %s %s");
             (void) fprintf(d->outfile,outFormat,
        		     d->move_buffer,"...",d->black_buffer);
             curCol +=strlen(d->move_buffer)+strlen(d->black_buffer)+7;
             break;
        }
        d->intrupt = FALSE ;
  } else {
  /* else (no intrupt)
     we display either white or black move
     */
    switch (d->type) {
    case D_TEX:
      if (d->iswhiteturn)
	(void) fprintf(d->outfile,"\\ %s\\ %s",d->move_buffer,d->white_buffer);
      else
	(void) fprintf(d->outfile,"\\ %s",d->black_buffer);
      break;
    case D_PS:
      if (d->iswhiteturn)
	 (void) fprintf(d->outfile,"( ) show %s ( ) show %s \n",d->move_buffer,d->white_buffer);
      else
 	(void) fprintf(d->outfile,"( ) show %s \n",d->black_buffer);
      curCol +=2;
      break;
    case D_XCHESS:
      if (d->iswhiteturn)
	(void) fprintf(d->outfile,"%3s.",d->move_buffer);
    case D_GNU:
      if (d->iswhiteturn)
	(void) fprintf(d->outfile,"%s",d-> white_buffer);
      else
	(void) fprintf(d->outfile," %s",d->black_buffer);
      break;
    case D_CHESSARC:
         if (d->iswhiteturn)
            (void) fprintf(d->outfile,"%-6s",d-> white_buffer);
          else
            (void) fprintf(d->outfile," %-6s\n",d->black_buffer);
         break;
    case D_ZARKOV:
      if (d->iswhiteturn)
	(void) fprintf(d->outfile," %s",d-> white_buffer);
      else
	(void) fprintf(d->outfile," %s",d->black_buffer);
      break;
    case D_CM:
      { unsigned char hibyte,lobyte;
      if (d->iswhiteturn)
        { CMconvert(d->white_buffer,&lobyte,&hibyte);
	  (void) fprintf(d->outfile,"%c%c%c%c",hibyte,lobyte,0x2,0x0);
          CMTurnCnt(WHITE);
        }
      else
        { CMconvert(d->black_buffer,&lobyte,&hibyte);
         (void) fprintf(d->outfile,"%c%c%c%c",hibyte,lobyte,0x2,0x0);
         CMTurnCnt(BLACK);
        }
      }
      break;
    default:
      if (d->iswhiteturn)
        {
         strcpy(outFormat,d->column?"%3s.%9s":" %s. %s");
         (void) fprintf(d->outfile,outFormat,d->move_buffer,d->white_buffer);
         curCol +=strlen(d->move_buffer)+strlen(d->white_buffer)+3;
        }
      else
       {
        strcpy(outFormat,d->column?"%9s":" %s");
	(void) fprintf(d->outfile,outFormat,d->black_buffer);
        curCol +=strlen(d->black_buffer)+1;
       }
      break;
    }
  } /* end printing */

  /* reset buffer */
  if (! d->iswhiteturn)
    init_buffer(d,VOID);
  d->intrupt = FALSE;


}

/* a generic parametrised driver for move output
   */
#ifdef __STDC__
static void output_move_generic(format *dr, depl *d)
#else
static void output_move_generic(dr,d)
     format * dr;
     depl *d;
#endif
{
  char ligne[128] ;
  char themove[128] ;
  char thepiece[16]  ;
  char debcol[16];
  char frommove[16]  ;
  char tomove[16] ;
  char captsymb[8] ;
  char lie[8] ;
  char prom[40];

  ligne[0] = themove[0] = thepiece[0] = debcol[0]  = '\0';
  frommove[0] = tomove[0] = lie[0] = prom[0] = '\0' ;

  if (dr->type == D_TEX)
    (void) sprintf(captsymb,"%s", FigCapt);
  else
    (void) sprintf(captsymb,"%s", "x" );

  /* print the move number */
  if(dr->type==D_PS)
    { (void) sprintf(dr->move_buffer,"B (%d) show ",d->move); curCol+=3; }
  else
    (void) sprintf (dr->move_buffer,"%d",d->move);

  if ((d->type == PETITROQUE) && !dr->roque_alg)
     if(dr->type==D_PS)
       { sprintf(themove," B (%s) show ",P_ROQUE); curCol+=3; }
      else
       (void) sprintf (themove,"%s",P_ROQUE);
  if ((d->type == GRANDROQUE) && !dr->roque_alg)
     if(dr->type==D_PS)
       { sprintf(themove," B (%s) show ",G_ROQUE); curCol+=5; }
      else
       (void) sprintf (themove,"%s",G_ROQUE);
  if (dr->roque_alg &&
      ((d->type == GRANDROQUE) || (d->type == PETITROQUE)))
    (void) roque_to_move(d);
  if (dr-> roque_alg ||
      ((d->type != GRANDROQUE) && (d->type != PETITROQUE)))
   {
      themove[0] = '\0' ;
      if ((dr->output_move_format == SHORTENED)
          && (d->type != MOVE)
 	  && (d->type != PROMOTION)
	  && (d->piece == PAWN))
             (void) sprintf (debcol, "%c",coltoletter(d->fromcol));

    if (dr->print_piece)
      if (d->piece != PAWN || dr->print_pawn)
      {
        if (dr->type == D_TEX )
          (void) sprintf(thepiece,"%s",latex_table[d->piece]);
        else
          (void) sprintf(thepiece,"%c",dr->out_table[d->piece]);
      }

    if ((dr->output_move_format == ALGEBRAIC))
      (void)sprintf(frommove,"%c%c",
		    coltoletter(d->fromcol),ligtoletter(d->fromlig));

    if (dr->print_liaison && (d->ambfile || d->ambrank))
       {
        if (d->ambfile) /* Ambiguous file */
           (void)sprintf(frommove,"%c", coltoletter(d->fromcol));
        if (d->ambrank) /* Ambigous rank */
           (void)sprintf(frommove,"%c", ligtoletter(d->fromlig));
        if(d->ambfile && d->ambrank)  /* Very Rare */
           (void)sprintf(frommove,"%c%c",coltoletter(d->fromcol),
                                         ligtoletter(d->fromlig));
        debcol[0] = '\0' ;
      }

    if (d->promotion) {
      if (dr->print_liaison) {
        if (dr->type == D_TEX )
          (void) sprintf(prom,"=%s ",latex_table[d->promotion]);
        else if (dr->type == D_PS)
          (void) sprintf(prom,"B (=) show C (%c) show",dr->out_table[d->promotion]);
          else
            (void) sprintf(prom,"=%c",dr->out_table[d->promotion]);
      } else /* xchess - gnu, Zarkov, CM output */
	(void) sprintf(prom,"%c",dr->out_table[d->promotion]);
    }

    if (dr->print_liaison) {
      if ((d->type == PRISE) || (d->type == PROM_ET_PRISE)
	  || (d->type == EN_PASSANT) )
	(void) sprintf(lie,"%s",captsymb);
      else
	if ((dr->output_move_format == ALGEBRAIC))
	  (void) sprintf(lie,"%c",'-');
    }

    (void) sprintf(tomove,"%c%c",coltoletter(d->tocol),ligtoletter(d->tolig));
    if(dr->type==D_PS)
      {
       (void) sprintf(themove,"C (%s) show B (%s%s%s%s) show %s",
                     thepiece,debcol,frommove,lie, tomove,prom);
       curCol +=(strlen(thepiece)+strlen(debcol)+strlen(frommove)+
                 strlen(lie)+strlen(tomove));
       if(strcmp(prom,"")!=0) curCol +=2;
      }
    else
    (void) sprintf (themove,"%s%s%s%s%s%s",
		    thepiece,debcol,frommove,lie, tomove,prom);
  }

  if (d->whiteturn)
    (void) sprintf (dr->white_buffer, "%s",themove);
  else
    (void) sprintf (dr->black_buffer, "%s",themove);

  dr->iswhiteturn = d->whiteturn;

  flush_buffer(dr);
  /* Add + if King in Check or # if mate */
  if(d->is_check)
     printCheck(d->is_check==MATE);

}

/* variation handler */
#ifdef __STDC__
static void output_variation_generic (format *dr, int inout)
#else
static void output_variation_generic (dr,inout)
     format * dr;
     int inout;
#endif
{
  char symbol;
  char outFormat[20];

  if (dr->variation > 1)
    symbol = varsymb[1][inout];
  else
    symbol = varsymb[0][inout];

  switch (dr->type) {
  case D_TEX:
    /* we must boldface the brackets for level 1 */
    if (dr->variation == 1 )
    {
      /*(void) fprintf(dr->outfile, " {\\bf %c} ",symbol);*/
      if (inout == 0 )
        (void) fprintf(dr->outfile, "%%\n\\begin{Variation}%%\n ");
      else
        (void) fprintf(dr->outfile, " \\end{Variation} %%\n");
    }
    else
     {
      strcpy(outFormat,(dr->column&&!inout)?"\n%c":" %c");
      (void) fprintf(dr->outfile,outFormat,symbol);
     }
    break;
  case D_PS:
    strcpy(outFormat,(dr->column&&!inout)?" R N (%c) show ":" R ( %c) show ");
    (void) fprintf(dr->outfile,outFormat,symbol);
    break;
  default:
    strcpy(outFormat,(dr->column&&!inout)?"\n%c":" %c");
    (void) fprintf(dr->outfile,outFormat,symbol);
    break;
  };
 curCol +=2;
}

#ifdef __STDC__
static void output_text_generic(format *dr, int type, char *string, int code)
#else
static void output_text_generic(dr, type, string, code)
     format *dr ;
     int type;
     char * string;
     int code;
#endif
{
  switch (type) {
  case T_COMMENT:
    curCol +=strlen(string);
    if (com_short[code] != '\0' )
      (void) fprintf(dr->outfile,"%s",com_short[code]);
    else
      (void) fprintf(dr->outfile,"%s",com_long[code]);
      break;
  case T_TEXT:
    (void) fprintf(dr->outfile," %s\n",string);
     dr->newline=TRUE;
     dr->intrupt=TRUE;
    break;
  case T_TITLE:
    (void) fprintf(dr->outfile,"\n  %s\n",string);
    break;
  case T_SUBTITLE:
    (void) fprintf(dr->outfile,"    %s\n",string);
    break;
  case T_SCORE:
    (void) fprintf(dr->outfile," %s\n",string);
    break;
  default:
    break;
  }

}

#ifdef __STDC__
static void output_endgame_generic(format *dr)
#else
static void output_endgame_generic()
            format *dr;
#endif
{
 if(dr->final_board)
    output_board(dr,tos);
 init_board(tos);
 m= theplay->chain;
 free_move_list(m);
}

/* ---------------- ascii driver ----------  */
#ifdef __STDC__
static void output_init_ascii(format *dr)
#else
static void output_init_ascii(dr)
format *dr;
#endif
{}

#ifdef __STDC__
static void output_board_ascii(format *dr,game *g)
#else
static void output_board_ascii(dr,g)
     format * dr;
     game * g;
#endif
{
  register int i,j;

  dr->newline=TRUE;
  dr->intrupt = TRUE;
  (void) fprintf(dr->outfile,"\n\n");
  for (i=8 ; i >=1 ; i--) {
    if (dr->coordinates)
      (void) fprintf(dr->outfile,"%d ",i);
    (void) fputc(ascboard[ASCBOARD_DELIM],dr->outfile);
    for (j=1 ; j<9 ; j++) {
      if (g->board[i][j] != VOID) {
	if (g->color[i][j] == WHITE)
	 (void) fputc(dr->out_table[g->board[i][j]], dr->outfile);
	else
	  (void) fputc(tolower(dr->out_table[g->board[i][j]]),dr->outfile);
      } else
	(void) fputc ( ((i+j)% 2)?ascboard[ASCBOARD_WHITE]:ascboard[ASCBOARD_BLACK], dr->outfile);
      (void) fputc(ascboard[ASCBOARD_DELIM], dr->outfile);
    }
    (void) fputc('\n', dr->outfile);
  }
  if (dr->coordinates)
    (void) fprintf(dr->outfile,"   a b c d e f g h\n");
  (void) fprintf(dr->outfile,"\n");
}
/* -------------- postscript - Walker Font ------------- */
#ifdef __STDC__
static void output_init_wps(format *dr)
#else
static void output_init_wps(dr)
format *dr;
#endif
{
  register int c;
  char chaine[MAXTOKLEN];

  /* header file */
  (void) strcpy(chaine,LIB_DIR);
  if ((ftmp = fopen((const char *)strcat(chaine,WPS_HEADER),"r")) == NULL)
    message((stderr,"Can't open Walker ps header file: %s\n",chaine));
  else {
    while ((c = getc(ftmp)) != EOF)
      (void) fputc(c,dr->outfile);
    (void) fclose(ftmp);
  }
}
#ifdef __STDC__
static void output_end_wps(format *dr)
#else
static void output_end_wps(dr)
format *dr;
#endif
{
  register int c;
  char chaine[MAXTOKLEN];

  /* footer file */
  (void) strcpy(chaine,LIB_DIR);
  if ((ftmp = fopen((const char *)strcat(chaine,WPS_FOOTER),"r")) == NULL)
    message((stderr,"Can't open Walker ps footer file: %s\n",chaine));
  else {
    while ((c = getc(ftmp)) != EOF)
      (void) fputc(c,dr->outfile);
    (void) fclose(ftmp);
  }

}


#ifdef __STDC__
static void output_text_wps(format *dr, int type, char *string, int code)
#else
static void output_text_wps(dr, type, string, code)
     format *dr ;
     int type;
     char * string;
     int code;
#endif
{
  char *outBuf;
  int mallocErr=0,i;
  if((outBuf = (char *)malloc(sizeof(char)*strlen(string)))==NULL)
    { outBuf=string; mallocErr=1; }
  else
   { strcpy(outBuf,string);
     for(i=0;i<strlen(outBuf);++i)
        if((outBuf[i]=='['))
         { outBuf[i]=' '; break; }
        else if ((outBuf[i]=='('))
         { outBuf[i]=' '; break; }
     for(i=strlen(outBuf);i>=0;--i)
        if((outBuf[i]==')') || (outBuf[i]==']'))
         { outBuf[i]=' '; break; }
   }
  switch (type) {
  case T_COMMENT:
    curCol +=2;
    if (com_short[code] != '\0' )
      (void) fprintf(dr->outfile," %s show ",com_ps[code]);
    else
      (void) fprintf(dr->outfile," %s show ",com_short[code]);
      break;
  case T_TEXT:
    (void) fprintf(dr->outfile," R ( ");
    for(i=0;i<strlen(outBuf);++i)
        if(outBuf[i]=='\n') fprintf(dr->outfile,") show N \n R (");
        else if (outBuf[i]=='\t') fprintf(dr->outfile,"        ");
        else if ((outBuf[i]=='(') || (outBuf[i]==')'))
                fprintf(dr->outfile,"%c%c",'\\',outBuf[i]);
        else fprintf(dr->outfile,"%c",outBuf[i]);
    fprintf(dr->outfile,") show N \n");
     dr->newline=TRUE;
     dr->intrupt=TRUE;
    break;
  case T_TITLE:
    (void) fprintf(dr->outfile,"N B (    %s) show N ",string);
    break;
  case T_SUBTITLE:
    (void) fprintf(dr->outfile,"B (    %s) show N ",string);
    break;
  case T_SCORE:
    (void) fprintf(dr->outfile,"B (    %s) show N ",string);
    break;
  default:
    break;
  }
  if(!mallocErr)
     free(outBuf);
}

#ifdef __STDC__
static void output_board_wps(format *dr,game *g)
#else
static void output_board_wps(dr,g)
     format *dr;
     game * g;
#endif
{
 int rank,file,spaceCnt;
 char boardBuf[80],temp[20];
 char pieces[]={' ','K','Q','R','B','N','P'};

 fprintf(dr->outfile,"/ps 19 def /vs 19 def \n"); /*almost same size as TeX*/
 fprintf(dr->outfile,"C N N N N N N N N N T\n");

 strcpy(boardBuf,"(\'#[");
 spaceCnt=0;
 for(rank=8;rank>0;--rank)
  {
   for(file=1;file<9;++file)
     {
      if(g->board[rank][file]==VOID)
          ++spaceCnt;
      else
        {
          if(spaceCnt!=0)
           { sprintf(temp,"%d",spaceCnt); strcat(boardBuf,temp); spaceCnt=0; }
          if(g->color[rank][file]==BLACK)
            { sprintf(temp,"%c",tolower(pieces[g->board[rank][file]]));
              strcat(boardBuf,temp);
            }
            else
             { sprintf(temp,"%c",pieces[g->board[rank][file]]);
               strcat(boardBuf,temp);
             }
        } /* else */
     } /* rank */
    if(spaceCnt!=0)
      { sprintf(temp,"%d",spaceCnt);
        strcat(boardBuf,temp); spaceCnt=0;
      }
    if(rank!=1) strcat(boardBuf,";");
   } /* file */
  strcat(boardBuf,"]\') show\n");
 fprintf(dr->outfile,"%s",boardBuf);
 fprintf(dr->outfile,"/ps 10.5 def /vs 12.5 def N N\n");

}
/* ---------------- postscript - Cheq -------- */

#ifdef __STDC__
static void output_board_ps(format *dr,game *g)
#else
static void output_board_ps(dr,g)
     format *dr;
     game * g;
#endif
{
  register int i,j;
  register int c;
  char chaine[MAXTOKLEN];

  /* header file */
  (void) strcpy(chaine,LIB_DIR);
  if ((ftmp = fopen((const char *)strcat(chaine,PS_HEADER),"r")) == NULL)
    message((stderr,"Can't open ps header file.\n"));
  else {
    while ((c = getc(ftmp)) != EOF)
      (void) fputc(c,dr->outfile);
    (void) fclose(ftmp);
  }

  (void) fprintf(dr->outfile,"( ________) 72 714 T\n");
  for (i=8 ; i >=1 ; i--) {
    (void) fprintf(dr->outfile,"(/");
    for (j=1 ; j<9 ; j++) {
    (void) fprintf(dr->outfile,"%s",
		   postscript_table[g->board[i][j]][PSINDEX(g->color[i][j],(i+j))]);
    }
    (void) fprintf(dr->outfile,"\\\\) 72 %d T\n",474 + (i-1)*30);
  }
  (void) fprintf(dr->outfile,"( --------) 72 444 T\n");

  /* footer file */
  (void) strcpy(chaine,LIB_DIR);
  if ((ftmp = fopen((const char *)strcat(chaine,PS_FOOTER),"r")) == NULL)
    message((stderr,"Can't open ps footer file.\n"));
  else {
    while ((c = getc(ftmp)) != EOF)
      (void) fputc(c,dr->outfile);
    (void) fclose(ftmp);
  }
}

/* ---------------- nroff --------------- */
#ifdef __STDC__
static void output_init_roff(format *dr)
#else
static void output_init_roff(dr)
     format *dr;
#endif
{
}

#ifdef __STDC__
static void output_board_roff(format *dr,game *g)
#else
static void output_board_roff(dr, g)
     format *dr;
     game * g;
#endif
{
  register int i,j;

  dr->newline=TRUE;
  dr->intrupt = TRUE;

  (void) fprintf(dr->outfile,".br\n");
  for (i=8 ; i >=1 ; i--) {
    (void) fprintf(dr->outfile,".ce\n  ");
    for (j=1 ; j<9 ; j++) {
      if (g->board[i][j] != VOID) {
	if (g->color[i][j] == WHITE)
	 (void) fputc(dr->out_table[g->board[i][j]], dr->outfile);
	else
	  (void) fputc(tolower(dr->out_table[g->board[i][j]]),dr->outfile);
      } else
	(void) fprintf(dr->outfile,".");
    }
    (void) fprintf(dr->outfile,"\n.br\n");
  }
  (void) fprintf(dr->outfile,"\n");
}

/* ---------------- tex -------------------- */
#ifdef __STDC__
static void output_init_tex(format *dr)
#else
static void output_init_tex(dr)
     format *dr;
#endif
{

  /* header text */
  fprintf(dr->outfile, "%% This file generated by the ChessPrn program\n");
  fprintf(dr->outfile, "%% @ Ed Maillet 1993\n");
  fprintf(dr->outfile, "\\documentstyle[twocolumn,chess]{article}\n");
  fprintf(dr->outfile, "\\input{notation.tex}\n");
  fprintf(dr->outfile, "\n");
  fprintf(dr->outfile, "\\begin{document}\n");
  fprintf(dr->outfile, "\n");
  fprintf(dr->outfile, "\\begin{Mainline}{}{}\n");
}

#ifdef __STDC__
static void output_text_tex(format *dr, int type, char * string, int code)
#else
static void output_text_tex(dr, type, string, code)
     format *dr ;
     int type;
     char * string;
     int code;
#endif
{
  char *outBuf;
  int mallocErr=0,i;

  if (type != T_COMMENT)
    (void) fprintf(dr->outfile, " \\nochess");

  if((outBuf = (char *)malloc(sizeof(char)*strlen(string)))==NULL)
    { outBuf=string; mallocErr=1; }
  else
   { strcpy(outBuf,string);
     fprintf(stderr,"Before:\n*%s*\n",outBuf);
     for(i=0;i<strlen(outBuf);++i)
        if(outBuf[i]=='[')
         { outBuf[i]=' '; break; }
        else if (outBuf[i]=='(')
         { outBuf[i]=' '; break; }
     for(i=strlen(outBuf)-1;i>0;--i)
        if(outBuf[i]==')' || outBuf[i]==']')
         { outBuf[i]=' '; break; }
    for(i=0;i<strlen(outBuf);++i)
        if(outBuf[i]=='\n') outBuf[i]=' ';
    fprintf(stderr,"After:\n*%s*\n",outBuf);
             
   }
  switch (type) {
  case T_COMMENT:
    if (com_tex[code] != '\0' )
      (void) fprintf(dr->outfile,"%s",com_tex[code]);
    else
      (void) fprintf(dr->outfile,"%s",com_short[code]);
      break;
  case T_TEXT:
    (void) fprintf(dr->outfile,"\n\n{\\rm %s}\n\n",outBuf);
     dr->newline=TRUE;
     dr->intrupt=TRUE;
    break;
  case T_TITLE:
    (void) fprintf(dr->outfile,"\n\\ChessTitle{%s}\n",outBuf);
    break;
  case T_SUBTITLE:
    (void) fprintf(dr->outfile,"\n\\ChessSubTitle{%s}\n",outBuf);
    break;
  case T_SCORE:
    (void) fprintf(dr->outfile,"\n\\ \\ChessScore{%s}\n",outBuf);
    break;
  default:
    break;
  };
  if (type != T_COMMENT)
    (void) fprintf(dr->outfile, "\\endnochess ");
  if(!mallocErr)
     free(outBuf);
}

#ifdef __STDC__
static void output_board_tex(format *dr,game *g)
#else
static void output_board_tex(dr,g)
     format *dr;
     game * g;
#endif
{
  register int i,j;

  dr->newline=TRUE;
  dr->intrupt = TRUE;

  (void) fprintf(dr->outfile,"\n\n\\begin{diagram}\n");
  (void)fprintf(dr->outfile,"\\board");
  for (i=8 ; i >=1 ; i--) {
    (void) fprintf(dr->outfile,"\t{");
    for (j=1 ; j < 9 ; j++) {
    (void) fprintf(dr->outfile,"%s",
		   texboard_table[g->board[i][j]][PSINDEX(g->color[i][j],(i+j))]);
    }
    (void) fprintf(dr->outfile,"}\n");
  }
  (void) fprintf(dr->outfile,"\\end{diagram}\n\n");
}

#ifdef __STDC__
static void output_end_tex(format *dr)
#else
static void output_end_tex(dr)
     format *dr;
#endif
{
  (void) fprintf(dr->outfile, "\n\\end{Mainline}\n");
  (void) fprintf(dr->outfile, " \n\n\n\\end{document}\n");
}

/* ------------------ gnu - xchess ---------- */

#ifdef __STDC__
static void output_init_gnu(format *dr)
#else
static void output_init_gnu(dr)
     format *dr;
#endif
{
  (void) fprintf(dr->outfile, "X Chess -- Mon Dec 10 11:47:18 MET 1990\n");
  (void) fprintf(dr->outfile,"\tGame played on super.machine:0.0\n");
  (void) fprintf(dr->outfile,"\talgebraic\n");
}
#ifdef __STDC__
static void output_endgame_xchess(format *dr)
#else
static void output_endgame_xchess(dr)
     format *dr;

#endif
{
  close_files();
  exit(0);
}

/* ------------------ Zarkov -------------------- */
/* Zarkov format - added by Ed Maillet */
/* If the board is setup this will be overwritten
   with the board postion
*/
#ifdef __STDC__
static void output_init_zarkov(format *dr)
#else
static void output_init_zarkov(dr)
     format *dr;
#endif
{
  int i;
  (void) fprintf(dr->outfile,"@ Zarkov.ntn\n");
  (void) fprintf(dr->outfile,"RNBQKBNR\n");
  (void) fprintf(dr->outfile,"PPPPPPPP\n");
  for(i=0;i<4;++i)
     (void) fprintf(dr->outfile,"--------\n");
  (void) fprintf(dr->outfile,"pppppppp\n");
  (void) fprintf(dr->outfile,"rnbqkbnr\n");
  (void) fprintf(dr->outfile,"WHITE\n");
}

/* Zarkov format - added by Ed Maillet */
#ifdef __STDC__
static void output_end_zarkov(format *dr)
#else
static void output_end_zarkov(dr)
     format *dr;
#endif
{
  (void) fprintf(dr->outfile,"\n"); /* close off ply list */
  (void) fprintf(dr->outfile,"T/C=0/0\n");
  (void) fprintf(dr->outfile,"WCLOCK=0\n");
  (void) fprintf(dr->outfile,"BCLOCK=0\n");
}

#ifdef __STDC__
static void output_endgame_zarkov(format *dr)
#else
static void output_endgame_zarkov(dr)
     format *dr;
#endif
{
  output_end_zarkov(dr);
  close_files();
  exit(0);
}
/* setup_zarkov_board:
    Overwrite default start position with given setup position
    Parameters:
       format *dr - output format
       game *pos  - current position
       int whiteturn - TRUE if white to move
  -- Ed Maillet
*/
#ifdef __STDC__
void setup_zarkov_board(format *dr, game *pos,int whiteturn)
#else
void setup_zarkov_board(dr,pos)
     format *dr;
     game *pos;
     int whiteturn;
#endif
{
  int rank,file,t;
  char outch;

  fseek(dr->outfile,-87,SEEK_CUR);
  for(rank=8;rank>0;--rank)
   {
    for(file=1;file<9;++file)
      {
       if((t=pos->board[rank][file])!=VOID)
         {
          outch=GetLangChar(ENGLISH,t);
          outch=(pos->color[rank][file]==BLACK)?toupper(outch):tolower(outch);
         }
       else  outch='-';
       (void) fprintf(dr->outfile,"%c",outch);
      }
     fprintf(dr->outfile,"\n");
    }
  fprintf(dr->outfile,"%s\n",whiteturn?"WHITE":"BLACK");

}
/* ----------------------- CM2100 & CM3000 formats ------------------- */
/* CM2100 & CM3000 format - added by Ed Maillet */
/* If the board is setup this will be overwritten
   with the board postion
*/
/* CMMapPiece - Map #define piece type to CM storage type

   Parameters:
     int piece - piece to be mapped
   Return:
     (char) CM storage of piece

*/
#ifdef __STDC__
static char CMMapPiece(int piece)
#else
static char CMMapPiece(piece)
     int piece;
#endif
{
/* char *CMMap[]={0x0,0x1,0x2,0x3,0x4,0x5};*/
 return (piece-1); /*CMMap[piece-1];*/
}
#ifdef __STDC__
static char CMconfig(int flag)
#else
static char CMconfig(flag)
     int flag;
#endif
{
 static char status=0;
 if(flag==CHECK)
    return status;
 status=SET;
 return status;
}
#ifdef __STDC__
static char CMTurnCnt(int flag)
#else
static char CMTurnCnt(flag)
     int flag;
#endif
{
  static plycnt; /* number of ply outputted */
  if(flag==CHECK)
     return plycnt;
  ++plycnt;
  return plycnt;
}

/* CMconvert - convert a buffer to a CM move */
#ifdef __STDC__
static void CMconvert(char *buffer,unsigned char *lobyte,unsigned char *hibyte)
#else
static void CMconvert(buffer)
     char *buffer;
     unsigned char *lobyte;
     unsigned char *hibyte;
#endif
{
 char fromFile,fromRank,toFile,toRank,prom;
 unsigned int move=0x0000;

 prom=0x7;
 fromFile=buffer[0]-'a';
 fromRank='8'-buffer[1];
 toFile=buffer[2]-'a';
 toRank='8'-buffer[3];
 if(buffer[4]!='\0') /* promotion */
    switch(buffer[4])
    { default: prom=CMMapPiece(QUEEN); break;
      case 'R': prom=CMMapPiece(ROOK); break;
      case 'B': prom=CMMapPiece(BISHOP); break;
      case 'N': prom=CMMapPiece(KNIGHT); break;
    }
  /* don't blink */
  move=toFile;
  move |=(toRank<<3);
  move |=(fromFile<<6);
  move |=(fromRank<<9);
  move |=(prom<<12);

  /* reverse move bytes */
  *lobyte=((move>>8) & 0x00FF);
  *hibyte=(move & 0xFF);
}

#ifdef __STDC__
static void output_init_cm(format *dr)
#else
static void output_init_cm(dr)
     format *dr;
#endif
{
 char *cmFileName={"ChessPrn Output"};
 char *whiteName={"Player"};
 char *blackName={"Other Player"};
 int i;

 if(strcmp(EventString,"?")!=0)
    cmFileName=EventString;
 if(strcmp(WhiteString,"?")!=0)
    whiteName=WhiteString;
 if(strcmp(BlackString,"?")!=0)
    blackName=BlackString;

 if(strlen(cmFileName)>31)
   cmFileName[31]='\0';
 if(strlen(whiteName)>(0x10-1))
    whiteName[0x10-1]='\0';
 if(strlen(blackName)>(0x10-1))
    blackName[0x10-1]='\0';

 fprintf(dr->outfile,"%s",cmFileName);
 for(i=0;i<32-strlen(cmFileName);++i)
    fputc(0x0,dr->outfile);
 fprintf(dr->outfile,"%c%c",0x1A,0x10);

 fprintf(dr->outfile,"%s",whiteName);
 for(i=0;i<16-strlen(whiteName);++i)
    fputc(0x0,dr->outfile);

 fprintf(dr->outfile,"%s",blackName);
 for(i=0;i<16-strlen(blackName);++i)
    fputc(0x0,dr->outfile);

 fprintf(dr->outfile,"%c%c",0x0,0xFF);
 fprintf(dr->outfile,"%c%c%c%c",0x00,0x00,0x00,0x00); /* Need to be overwritten later */

}

#ifdef __STDC__
static void output_end_cm(format *dr)
#else
static void output_end_cm(dr)
     format *dr;
#endif
{
  int config;
  char plycnt;
  long pos;

  /* Go back and adjust data */
  pos=ftell(dr->outfile);
  fseek(dr->outfile,0,SEEK_SET);
  output_init_cm(dr);
  fseek(dr->outfile,pos,SEEK_SET);

  config=CMconfig(CHECK);
  if(config)
    fseek(dr->outfile,132,SEEK_SET);
   else
    fseek(dr->outfile,68,SEEK_SET);

  plycnt=CMTurnCnt(CHECK);
  fprintf(dr->outfile,"%c%c%c%c",plycnt,0x0,plycnt,0x0);
  fseek(dr->outfile,0L,SEEK_END);

}
#ifdef __STDC__
static void output_endgame_cm(format *dr)
#else
static void output_endgame_cm(dr)
     format *dr;
#endif
{
  output_end_cm(dr);
  close_files();
  exit(0);
}


/* setup_cm_board:
    Overwrite default start position with given setup position
    Parameters:
       format *dr - output format
       game *pos  - current position
       int whiteturn - TRUE if white to move
  -- Ed Maillet
*/
#ifdef __STDC__
void setup_cm_board(format *dr, game *pos,int whiteturn)
#else
void setup_cm_board(dr,pos)
     format *dr;
     game *pos;
     int whiteturn;
#endif
{
  int rank,file,t;
  char outch;

  CMconfig(SET);
  fseek(dr->outfile,-5,SEEK_CUR);
  fprintf(dr->outfile,"%c",whiteturn?0x10:0x20);

  for(rank=8;rank>0;--rank)
    for(file=1;file<9;++file)
      {
       if((t=pos->board[rank][file])!=VOID)
         {
          outch=CMMapPiece(t);
          outch |=(pos->color[rank][file]==BLACK)?0x20:0x10;
         }
       else  outch=0x00;
      (void) fprintf(dr->outfile,"%c",outch);
     }
 fprintf(dr->outfile,"%c%c%c%c",0x00,0x00,0x00,0x00); /* Need to be overwritten later */

}
/* ----- PGN (Portable Game Notation) --------- */
#ifdef __STDC__
void SetPGNString(int type,char *inBuf)
#else
void SetPGNString(type,inBuf)
     int type;
     char *inBuf;
#endif
{
 int i,j;
 char outBuf[128],temp[128];
 for(i=0;i<128;++i)
    outBuf[i]='\0';
 strncpy(temp,inBuf,sizeof(temp));
 for(i=0,j=0;i<strlen(temp);++i)
    if(temp[i]!='\"') temp[j++]=temp[i];
 temp[j]='\0';
 switch(type)
  {
   case PGN_EVENT:
        strncpy(EventString,temp,sizeof(EventString));
        sprintf(outBuf,"[Event \"%s\"]",EventString);
        parse_text(outBuf);
        break;
   case PGN_SITE:
        strncpy(SiteString,temp,sizeof(SiteString));
        sprintf(outBuf,"[Site \"%s\"]",SiteString);
        parse_text(outBuf);
        break;
   case PGN_DATE:
        strncpy(DateString,temp,sizeof(DateString));
        sprintf(outBuf,"[Date \"%s\"]",DateString);
        parse_text(outBuf);
        break;
   case PGN_ROUND:
        strncpy(RoundString,temp,sizeof(RoundString));
        sprintf(outBuf,"[Round \"%s\"]",RoundString);
        parse_text(outBuf);
        break;
   case PGN_WHITE:
        strncpy(WhiteString,temp,sizeof(WhiteString));
        sprintf(outBuf,"[White \"%s\"]",WhiteString);
        parse_text(outBuf);
        break;
   case PGN_BLACK:
        strncpy(BlackString,temp,sizeof(BlackString));
        sprintf(outBuf,"[Black \"%s\"]",BlackString);
        parse_text(outBuf);
        break;
   case PGN_RESULT:
         if(strcmp(temp,"1/2-1/2")==0 || strcmp(temp,"1/2")==0)
            strcpy(temp,"1/2-1/2");
         else if(strcmp(temp,"0-1")==0 || strcmp(temp,"0:1")==0)
            strcpy(temp,"0-1");
         else if(strcmp(temp,"1-0")==0 || strcmp(temp,"1:0")==0)
            strcpy(temp,"1-0");
         else /* check with SJE on this one */
            strcpy(temp,"1-1"); /* unknown outcome */

       strncpy(ResultString,temp,sizeof(ResultString));
       sprintf(outBuf,"[Result \"%s\"]",ResultString);
       parse_text(outBuf);
        break;
   case PGN_COMMENT:
        strncpy(CommentString,temp,sizeof(CommentString));
        sprintf(outBuf,"[Comment \"%s\"]",CommentString);
        parse_text(outBuf);
        break;
   default:
        fprintf(stderr,"SetPGNString: Unhandled Case: %d\n",type);
        break;
  }


}
#ifdef __STDC__
static void FlushPGNStrings(int type)
#else
static void FlushPGNStrings()
       int type;
#endif
{
  int i;
  char outBuf[128];
  char *quote;
  char head[10],end[5];
  char *title[]={"[Event ","[Site ","[Date ","[Round ",
                 "[White ","[Black ","[Result "};
  char *str[]={EventString,SiteString,DateString,RoundString,WhiteString,
               BlackString,ResultString};

  for(i=0;i<7;++i)
   {
    if(type==D_PGN)
      { strcpy(head,title[i]); strcpy(end,"]"); quote="\"";}
    else
     { strcpy(head,""); strcpy(end,""); quote="";}
    sprintf(outBuf,"%s%s%s%s%s",head,quote,str[i],quote,end);
    parse_pgn_text(outBuf);
   }
  if(type==D_CHESSARC)
      parse_pgn_text(CommentString);

}

#ifdef __STDC__
void InitPGNString(void)
#else
void InitPGNString()
#endif
{
 strcpy(EventString,"?");
 strcpy(SiteString,"?");
 strcpy(DateString,"????.??.??");
 strcpy(RoundString,"?");
 strcpy(WhiteString,"?");
 strcpy(BlackString,"?");
 strcpy(ResultString,"1-1");
 strcpy(CommentString,"?");

}
#ifdef __STDC__
static void output_endgame_pgn(format *dr)
#else
static void output_endgame_pgn()
     format *dr;
#endif
{
 char outBuf[20];
 int i,j;
 FlushPGNStrings(dr->type);
 dr->out_move = output_move_generic; /* sneaky */
 fprintf(dr->outfile,"\n");
 print_all_play(theplay);
 dr->out_move=null_driver;
 for(i=0,j=0;i<20 && ResultString[i]!='\0';++i)
    if (ResultString[i]!='\"') outBuf[j++]=ResultString[i];
 outBuf[j]='\0';
 fprintf(dr->outfile," %s\n\n",outBuf);
 InitPGNString();
 init_board(tos);
 m= theplay->chain;
 free_move_list(m);

}
/* ----- ChessArc internal */
#ifdef __STDC__
static void output_endgame_chessarc(format *dr)
#else
static void output_endgame_chessarc(dr)
     format *dr;
#endif
{
 char outBuf[20];
 int i,j;
 FlushPGNStrings(dr->type);
 dr->out_move = output_move_generic; /* sneaky */
 print_all_play(theplay);
 dr->out_move=null_driver;
 for(i=0,j=0;i<20 && ResultString[i]!='\0';++i)
    if (ResultString[i]!='\"') outBuf[j++]=ResultString[i];
 outBuf[j]='\0';
 fprintf(dr->outfile,"\n%s\n",outBuf);
 InitPGNString();
 init_board(tos);
 m= theplay->chain;
 free_move_list(m);

}

/* ---------------- driver handler ---------- */
/* dummy driver */
#ifdef __STDC__
static void null_driver(void) {}
#else
static void null_driver() {}
#endif

/* the drivers */
#ifdef __STDC__
void output_init(format *dr)
#else
void output_init(dr)
     format *dr ;
#endif
{
  if (dr->print_headers)
    dr->out_init(dr);
}

#ifdef __STDC__
void output_move(format *dr,depl *d)
#else
void output_move(dr,d)
     format *dr ;
     depl *d;
#endif
{
  if (! (((dr->type == D_GNU) || (dr->type == D_XCHESS))
	&& (dr->variation > 0)))
    dr->out_move(dr,d);
}

#ifdef __STDC__
void output_variation(format *dr, int inout)
#else
void output_variation(dr, inout)
     format *dr ;
     int inout;
#endif
{
  dr->out_variation(dr,inout);
}

#ifdef __STDC__
void output_text(format *dr, int type, char *string, int code,int override)
#else
void output_text(dr, type, string, code)
     format *dr ;
     int type;
     char * string;
     int code;
     int override;
#endif
{
  if(override)
     output_text_generic(dr,type,string,code);
  else if ((dr->type != D_GNU) && (dr->type != D_XCHESS))
    dr->out_text(dr, type, string, code);
}

#ifdef __STDC__
void output_board(format *dr, game *g)
#else
void output_board(dr,g)
     format *dr ;
     game *g ;
#endif
{
  dr->out_board(dr,g);
}

#ifdef __STDC__
void output_end(format *dr)
#else
void output_end(dr)
     format *dr ;
#endif
{
  if(dr->ingame)
     dr->out_end_game(dr);
  if (dr->print_headers)
    dr->out_end_file(dr);
  if(dr->type!=D_ZARKOV && dr->type!=D_CM)
    (void) fprintf(dr->outfile,"\n");
}
#ifdef __STDC__
void output_end_game(format *dr)
#else
void output_end_game(dr)
     format *dr ;
#endif
{
 dr->out_end_game(dr);
 dr->ingame=FALSE;
}


#ifdef __STDC__
format * new_driver(void)
#else
format * new_driver()
#endif
{
  format * tmp;
  int i;

  tmp = (format *) malloc (sizeof(format));
  ALLOCP(tmp);
  for (i=0; i < ((sizeof (format))/ sizeof (int)) ; i++)
    ((int *) tmp)[i] = 0;
  tmp->output_move_format = SHORTENED;
  tmp->print_headers = TRUE;
  tmp->outfile = stdout;
  tmp->final_board=TRUE;
  tmp->column=FALSE;
  tmp->ingame=FALSE;

  return(tmp);
}

#ifdef __STDC__
void init_driver(format *dr,int driver)
#else
void init_driver(dr,driver)
     format * dr;
     int driver;
#endif
{
  dr->type = driver ;

  init_buffer(dr, VOID);
  switch (dr->type) {
  case D_ASCII:
    dr->print_move = TRUE;
    dr->print_piece = TRUE;
    dr->print_pawn = FALSE;
    dr->roque_alg = FALSE;
    dr->print_liaison = TRUE;
    dr->out_init = output_init_ascii;
    dr->out_move = output_move_generic;
    dr->out_variation = output_variation_generic;
    dr->out_text = output_text_generic;
    dr->out_board = output_board_ascii;
    dr->out_end_file = null_driver;
    dr->out_end_game = output_endgame_generic;

    /* specific ascii toggle */
    dr->coordinates = TRUE;
    break;

  case D_POST:
    dr->out_init = null_driver;
    dr->out_move = null_driver;
    dr->out_variation = null_driver;
    dr->out_text = null_driver;
    dr->out_board = output_board_ps;
    dr->out_end_file = null_driver;
    dr->out_end_game = output_endgame_generic;
    break;
  case D_TEX:
    associe_traduction (&(dr->out_table),ENGLISH);
    dr->print_move = TRUE;
    dr->print_piece = TRUE;
    dr->print_pawn = FALSE;
    dr->roque_alg = FALSE;
    dr->print_liaison = TRUE;
    dr->out_init = output_init_tex;
    dr->out_move = output_move_generic;
    dr->out_variation = output_variation_generic;
    dr->out_text = output_text_tex;
    dr->out_board = output_board_tex;
    dr->out_end_file = output_end_tex;
    dr->out_end_game = output_endgame_generic;
    break;
  case D_PS:
    associe_traduction (&(dr->out_table),ENGLISH);
    dr->print_move = TRUE;
    dr->print_piece = TRUE;
    dr->print_pawn = FALSE;
    dr->roque_alg = FALSE;
    dr->print_liaison = TRUE;
    dr->out_init = output_init_wps;
    dr->out_move = output_move_generic;
    dr->out_variation = output_variation_generic; /* not operational yet */
    dr->out_text = output_text_wps;
    dr->out_board = output_board_wps;
    dr->out_end_file = output_end_wps;
    dr->out_end_game = output_endgame_generic;
    break;
  case D_ROFF:
    dr->print_move = TRUE;
    dr->print_piece = TRUE;
    dr->print_pawn = FALSE;
    dr->roque_alg = FALSE;
    dr->print_liaison = TRUE;
    dr->out_init = output_init_roff;
    dr->out_move = output_move_generic;
    dr->out_variation = output_variation_generic;
    dr->out_text = output_text_generic;
    dr->out_board = output_board_roff;
    dr->out_end_file = null_driver;
    dr->out_end_game = output_endgame_generic;
    break;
  case D_XCHESS:
    dr->output_move_format = ALGEBRAIC;
    dr->print_move = TRUE;
    dr->print_piece = FALSE;
    dr->print_pawn = FALSE;
    dr->roque_alg = TRUE;
    dr->print_liaison = FALSE;
    dr->out_init = output_init_gnu;
    dr->out_move = output_move_generic;
    dr->out_variation = null_driver;
    dr->out_text = null_driver;
    dr->out_board = null_driver;
    dr->out_end_file = null_driver;
    dr->out_end_game = output_endgame_xchess;
    break;
  case D_GNU:
    dr->output_move_format = ALGEBRAIC;
    dr->print_move = FALSE;
    dr->print_piece = FALSE;
    dr->print_pawn = FALSE;
    dr->roque_alg = TRUE;
    dr->print_liaison = FALSE;
    dr->out_init = null_driver;
    dr->out_move = output_move_generic;
    dr->out_variation = null_driver;
    dr->out_text = null_driver;
    dr->out_board = null_driver;
    dr->out_end_file = null_driver;
    dr->out_end_game = output_endgame_generic;
    break;
  case D_ZARKOV: /* Ed Maillet */
    dr->output_move_format = ALGEBRAIC;
    dr->print_move = FALSE;
    dr->print_piece = FALSE;
    dr->print_pawn = FALSE;
    dr->roque_alg = TRUE;
    dr->print_liaison = FALSE;
    dr->out_init = output_init_zarkov;
    dr->out_move = output_move_generic;
    dr->out_variation = null_driver;
    dr->out_text = null_driver;
    dr->out_board = null_driver;
    dr->out_end_file = null_driver;
    dr->out_end_game = output_endgame_zarkov;
    break;
  case D_CM: /* Ed Maillet */
    InitPGNString();
    associe_traduction (&(dr->out_table),ENGLISH);
    dr->output_move_format = ALGEBRAIC;
    dr->print_move = FALSE;
    dr->print_piece = FALSE;
    dr->print_pawn = FALSE;
    dr->roque_alg = TRUE;
    dr->print_liaison = FALSE;
    dr->out_init = output_init_cm;
    dr->out_move = output_move_generic;
    dr->out_variation = null_driver;
    dr->out_text = null_driver;
    dr->out_board = null_driver;
    dr->out_end_file = output_end_cm;
    dr->out_end_game = output_endgame_cm;
    break;
  case D_PGN: /* Ed Maillet */
    InitPGNString();
    associe_traduction (&(dr->out_table),ENGLISH);
    dr->output_move_format = SHORTENED;
    dr->print_move = TRUE;
    dr->print_piece = TRUE;
    dr->print_pawn = FALSE;
    dr->roque_alg = FALSE;
    dr->print_liaison = TRUE;
    dr->out_init = output_init_ascii;
    dr->out_move = null_driver; /* over ridden later */
    dr->out_variation = null_driver;
    dr->out_text = null_driver;
    dr->out_board = null_driver;
    dr->out_end_file = null_driver;
    dr->out_end_game = output_endgame_pgn;
    dr->column=FALSE;
    break;
  case D_CHESSARC: /* Ed Maillet - ChessArc internal insert Format */
    InitPGNString();
    associe_traduction (&(dr->out_table),ENGLISH);
    dr->output_move_format = ALGEBRAIC;
    dr->print_move = FALSE;
    dr->print_piece = FALSE;
    dr->print_pawn = FALSE;
    dr->roque_alg = TRUE;
    dr->print_liaison = FALSE;
    dr->out_init = null_driver;
    dr->out_move = null_driver; /* over ridden later */
    dr->out_variation = null_driver;
    dr->out_text = null_driver;
    dr->out_board = null_driver;
    dr->out_end_file = null_driver;
    dr->out_end_game = output_endgame_chessarc;
    break;
  default:
    error((stderr,"unknown driver"));
    break;
  }
  if (dr->only_board)
    dr->out_move = null_driver ;

  dr->variation = 0;
  dr->iswhiteturn = FALSE ;
  dr->intrupt = FALSE;
  dr->newline=FALSE;
}
