/*
  ChessPrn program by Ed Maillet
  @(#)drivers.h	1.0 Dated 3/93
  Copyright (C) 1993  Ed Maillet
  Modified and based on Notation 4.7 by Henry Thomas
 */
/* headers for output drivers */
#ifndef _HEADERS_DRIVERS
#define HEADERS_DRIVERS


#define D_ASCII    0
#define D_POST     1 /* Adobe Cheq font */
#define D_TEX      2
#define D_ROFF     3
#define D_XCHESS   4
#define D_GNU      5
#define D_ZARKOV   6
#define D_CM       7
#define D_PGN      8
#define D_CHESSARC 9 /* ChessArc Internal */
#define D_PS      10 /* Walker Font */

#define NB_DRIVER 11

/* variation convention */
#define VARIATION_IN 0
#define VARIATION_OUT 1


/* output buffers */
#define TAMPON 256

/* customization of ascii board print */
/* string of chars used for the asc board = 3 differents chars */
#ifndef ASCBOARD_STRING
#define ASCBOARD_STRING "| /"
/* alternate string (a` la L. Blair) */
/*#define ASCBOARD_STRING " --"*/
#endif

/* first char is vertical delimiter */
#define ASCBOARD_DELIM  0
/* second char is white background */
#define ASCBOARD_WHITE  1
/* third char is black background  */
#define ASCBOARD_BLACK  2

typedef struct {

  /* type of driver */
  int type ;

  /* output_file */
  FILE *outfile ;
  /* these  booleans control the output format */
  int print_move    ;  /* move numbering */
  int print_piece   ;  /* print piece name */
  int print_pawn    ; /* print the PAWN name */
  int roque_alg     ; /*  roque in algebraic form Ke1g1 or O-O */
  int print_liaison ; /* print the - or x in move output */
  int only_board ;
  int variation     ; /* variation level */
  int print_headers ; /* include the header/footer file */
  int final_board;    /* print final board? */
  int ingame;         /* has a move been parsed since last end of game */

  /* boolean to print the coordinates in ascii output of board */
  int coordinates   ;

  /* format of the output move ( SHORTENED or complete ) */
  int output_move_format ;

  char *out_table;	/* translation table */

  /* procedures */
  void (*out_init)() ;
  void (*out_move)() ;
  void (*out_variation)() ;
  void (*out_text)() ;
  void (*out_board)() ;
  void (*out_end_file)() ;
  void (*out_end_game)();

  /* temp vars  used by move buffering */

  int iswhiteturn ; /*= FALSE */
  int intrupt ; /*= FALSE */
  int newline; /* TRUE if newline is to be printed before next move */
  int column;  /* TRUE if moves in column form else 1. e4 e5 2. d4 d5 etc. */

  char move_buffer[TAMPON]  /*= ""*/ ;
  char white_buffer[TAMPON] /*= ""*/ ;
  char black_buffer[TAMPON] /*= ""*/ ;


} format ;


/* fonctions ----------------- */

#ifdef __STDC__

extern void output_init(format *dr);
extern void output_move(format *dr, depl *d);
extern void output_variation(format *dr, int inout);
extern void output_text(format *dr, int type, char *string, int code,int override);
extern void output_board(format *dr, game *g);
extern void output_end(format *dr);
extern void output_end_game(format *dr);
extern format *new_driver(void);
extern void init_driver(format *dr, int driver);
extern void setup_zarkov_board(format *dr, game *pos,int whiteturn);
extern void setup_cm_board(format *dr, game *pos,int whiteturn);
extern void SetPGNString(int type,char *inBuf);
#else

extern void output_init(/*format *dr*/);
extern void output_move(/*format *dr, depl *d*/);
extern void output_variation(/*format *dr, int inout*/);
extern void output_text(/*format *dr, int type, char *string, int code, int override*/);
extern void output_board(/*format *dr, game *g*/);
extern void output_end(/*format *dr*/);
extern void output_end_game(/*format *dr*/);
extern format *new_driver(/*void*/);
extern void init_driver(/*format *dr, int driver*/);
extern void setup_zarkov_board(/*format *dr, game *pos,int whiteturn*/);
extern void setup_cm_board(/*format *dr, game *pos,int whiteturn*/);
extern void SetPGNString(/*int type,char *inBuf*/);
#endif

#endif
