/* DO NOT REMOVE THIS LINE                      */
/*                                              */
/* ChessPrn program by Ed Maillet               */
/* @(#)lexer.l 1.0 Dated 3/93                   */
/*                                              */
%{
#include <stdio.h>
#include <string.h>
/* Damn Borland */
#ifdef __TURBOC__
#define fileno(f)       ((f)->fd)
#endif

#include "chesstyp.h"
#include"notation.h"

#define NCURLINE 1024
char curline [NCURLINE] ;

#define LARGE_BUF 4096
static char commbuf[LARGE_BUF];

int column = 0;
int lineno = 1;
char filename[256];

#ifdef __STDC__
extern void count(void);
extern char * comment(char closing,char opening);
extern void parse_PGNinfo(char *token);
extern void SetPGNString(int type,char *inBuf);
#else
extern void count();
extern char * comment();
extern void parse_PGNinfo();
extern void SetPGNString()
#endif

%}

LEXSHORT [-+!?#]
LEXLONG  [-+=/\\\^\&\*\~<>_|#]
%%
[Oo0]\-?[Oo0](\-?[Oo0])? { /* roque */ ; parse_roque(yytext); }
[0-9]+[\.]?	        { /* move number */ parse_number(yytext); }
[A-Z]?[:/]?([a-z])?([1-8])?[-xX]?[a-z][1-8]?=?[A-Z]?  { /*move*/
   (void) parse_move(yytext); }
"1-0"|"1:0"|"1/2"|"1/2-1/2"|"0-1"|"0:1"|"1-1":"1:1" { /* game terminator */ parse_result(yytext); }
{LEXSHORT}{LEXSHORT}    { /* comment */ ; parse_comment(yytext); }
{LEXSHORT}              { /* comment */ ; parse_comment(yytext); }

{LEXLONG}{LEXLONG}{LEXLONG} { /* comment */ ;parse_comment(yytext); }
{LEXLONG}{LEXLONG}      { /* comment */ ; parse_comment(yytext); }
{LEXLONG}               { /* comment */ ; parse_comment(yytext); }
"\(\.\)"                { /* comment */ ; parse_comment(yytext); }
"\(\*\)"                { /* comment */ ; parse_comment(yytext); }
"\(\^\)"                { /* comment */ ; parse_comment(yytext); }
"\[\+\]"                { /* comment */ ; parse_comment(yytext); }
"\(\)"                  { /* comment */ ; parse_comment(yytext); }
"\[\]"                  { /* comment */ ; parse_comment(yytext); }
"o\.\.o"                { /* comment */ ; parse_comment(yytext); }
"oo"                    { /* comment */ ; parse_comment(yytext); }
"o/o"                   { /* comment */ ; parse_comment(yytext); }
"@\^"                   { /* comment */ ; parse_comment(yytext); }
"ep"|"e\.p\."           { /* comment */ ; parse_comment("ep"); }

\$[a-z][a-z][a-z]       { /* game comment */ ; parse_comment(yytext); }
@[a-z]+\{		{ /* keyword with arg */
                          yytext[yyleng -1] = '\0' ;
                          parse_keyword(yytext, comment('}','{'));
                        }
@[a-z]+			{ /* keyword without arg */
                          parse_keyword(yytext,NULL); }
"\["			{ parse_text(comment(']','[')); }
"\("			{ parse_text(comment(')','(')); }
"\{"                    { /* enter variation */ ; enter_variation(); }
"\}"			{ /* close variation */ ; exit_variation(); }
[\,;\.]			{ /* skip , ; */ ; }
[ \t\v\n\f]             { /* skip blanks */ count(); }
"[Event "|"[Site "|"[Date "|"[Round "|"[White "|"[Black "|"[Result "|"[Comment " {yytext[yyleng-1] = '\0' ; parse_PGNinfo(yytext); }
.                       { /* ignore bad characters */ (void) fprintf(stderr,"I don't understand: %s\n",yytext);}
%%

#ifdef FLEX_SCANNER
#undef yywrap
#endif
#ifdef __STDC__
int yywrap(void)
#else
int yywrap()
#endif
{
        return(1);
}

/* this procedure store comments in the text array commbuf
   */
#ifdef __STDC__
char * comment(char closing,char opening)
#else
char * comment(closing,opening)
     char closing;
     char opening;
#endif
{
  register char c;
  register int i=0;
  int opencnt=1; /* for nested comment symbols */

  strcpy(commbuf,"");
  if(closing!='}')
     strcpy(commbuf,(closing==']' ? "[" : "\n("));
  i=strlen(commbuf);
  while ( ((c=input()) != 0) && (c != EOF) && opencnt>0)
   {
    commbuf[i] = c;
    if(c==closing) {--opencnt;}
    else if(c==opening) {++opencnt;}
     else if (c=='\n') lineno++;
    if (i <LARGE_BUF-1) i++ ;
  }
 if(closing=='}')
    --i;
 if(closing==')')
     commbuf[i++]='\n';
  commbuf[i] = '\0' ;
  return(commbuf);
}

#ifdef __STDC__
void count(void)
#else
void count()
#endif
{
  register int i;
  register int k;

  for (i = 0; yytext[i] != '\0'; i++) {
    if (yytext[i] == '\n') {
      column = 0;
      for (k = 0 ; k< NCURLINE; k++)
        curline[k] = ' ';
      lineno++;
    }
    else if (yytext[i] == '\t') {
      column += 8 - (column % 8);
      curline[column] = yytext[i];
    } else {
      column++;
      curline[column] = yytext[i];
    }
    curline[column+1]= '\0' ;
  }
}

#ifdef __STDC__
void parse_PGNinfo(char *token)
#else
void parse_PGNinfo(token)
     char *token;
#endif
{
  char c;
  int i=0;
  char inBuf[1024];
  while ( ((c=input()) != 0) && (c != EOF) && (c!=']'))
       { inBuf[i++]=c; }
  inBuf[i]='\0';
  if(strcmp("[Event",token)==0)
     SetPGNString(PGN_EVENT,inBuf);
  else if(strcmp("[Site",token)==0)
     SetPGNString(PGN_SITE,inBuf);
  else if(strcmp("[Date",token)==0)
     SetPGNString(PGN_DATE,inBuf);
  else if(strcmp("[Round",token)==0)
     SetPGNString(PGN_ROUND,inBuf);
  else if(strcmp("[White",token)==0)
     SetPGNString(PGN_WHITE,inBuf);
  else if(strcmp("[Black",token)==0)
     SetPGNString(PGN_BLACK,inBuf);
  else if(strcmp("[Result",token)==0)
     SetPGNString(PGN_RESULT,inBuf);
  else if(strcmp("[Comment",token)==0)
     SetPGNString(PGN_COMMENT,inBuf);
  else /* shouldn't happen */
     fprintf(stderr,"Unknown PGN String: %s\n",token);
}
