/*
  ChessPrn program by Ed Maillet
  @(#)movedo.c	1.0 Dated 3/93
  Copyright (C) 1993  Ed Maillet
  Modified and based on Notation 4.7 by Henry Thomas
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "config.h"
#include "chesstyp.h"
#include "notation.h"

#include "movedo.h"


/* ---------- board management function ------------- */

#ifdef __STDC__
void clear_board(game *g)
#else
void clear_board(g)
     game *g;
#endif
{
  register int i,j;

  for (i=0; i < 10; i++ )
    for (j=0 ; j< 10 ; j++) {
      g->board[i][j] = VOID;
      g->color[i][j] = VOID;
    }
}

#ifdef __STDC__
game * new_board(void)
#else
game * new_board()
#endif
{
  game * tmp;
  int i; 

  tmp = (game *) malloc (sizeof(game));
  ALLOCP(tmp);
  for (i=0; i < ((sizeof (game))/ sizeof (int)) ; i++)
    ((int *) tmp)[i] = 0;
  return(tmp);
}

#ifdef __STDC__
game * copy_board(game *from, game *to)
#else
game * copy_board(from, to)
     game * from;
     game * to;
#endif
{
  int i; 

  for (i=0; i < ((sizeof (game))/ sizeof (int)) ; i++)
    ((int *) to)[i] =  ((int *) from)[i] ;
  return(to);
}

#ifdef __STDC__
void init_board(game *tgm)
#else
void init_board(tgm)
  game * tgm;
#endif
{
  register int i,j;

  clear_board(tgm);

  for (i=1; i< 9 ; i=i+7) {
    tgm->board[i][1]= tgm->board[i][8] = ROOK ;
    tgm->board[i][2]= tgm->board[i][7] = KNIGHT ;
    tgm->board[i][3]= tgm->board[i][6] = BISHOP ;
    tgm->board[i][4]= QUEEN;
    tgm->board[i][5]= KING;
  }
  for (i=2; i< 8 ; i=i+5) 
    for (j=1; j <=8 ; j++)
      tgm->board[i][j] = PAWN;

  for (i=1; i <=2; i++)
    for (j=1; j <=8 ; j++) {
      tgm->color[i][j] = WHITE;
      tgm->color[i+6][j] = BLACK ;
    }
}

#ifdef __STDC__
depl * new_move(void)
#else
depl * new_move()
#endif
{
  depl * tmp;
  int i; 
  static int counter = 0;

  tmp = (depl *) malloc (sizeof(depl));
  ALLOCP(tmp);
  for (i=0; i < ((sizeof (depl))/ sizeof (int)) ; i++)
    ((int *) tmp)[i] = 0;
  tmp->uid = ++counter;
  tmp->whiteturn = FALSE;
  tmp->move = 0;
  tmp->sub=NULL;
  tmp->next=NULL;
  tmp->prev=NULL;
  tmp->type=EMPTYNODE;
  return(tmp);
}


#ifdef __STDC__
void init_move(depl *m)
#else
void init_move(m)
     depl *m;
#endif
{
  m->move= 1 ;
  m->whiteturn = TRUE ;
}

#ifdef __STDC__
depl * copy_move(depl *from,depl *to)
#else
depl * copy_move(from,to)
     depl * from;
     depl * to ;
#endif
{
  int i; 

  for (i=0; i < ((sizeof (depl))/ sizeof (int)) ; i++)
    ((int *) to)[i] = ((int *) from)[i];

  return(to);
}

/* add a new move as successor to the move m */
#ifdef __STDC__
depl * add_trailing_move(depl *mo)
#else
depl * add_trailing_move(mo)
     depl * mo;
#endif
{
  mo->next = new_move();

  mo->next->prev = mo;
  mo->next->next = (depl *) NULL;
  mo->next->sub  = (depl *) NULL;

  mo->next->whiteturn = !(m->whiteturn);
  mo->next->move = mo->move;
  if (mo->next->whiteturn)
     mo->next->move++;


  return(mo->next);
}

#ifdef __STDC__
depl * add_variation(depl *mo)
#else
depl * add_variation(mo)
     depl * mo;
#endif
{
  depl *ip ; /* insertion point */

  ip = mo ;
  while (ip->sub != (depl *) NULL )
    ip = ip->sub;

  ip->sub = new_move();

  ip->sub->prev = mo;
  ip->sub->next = (depl *) NULL;
  ip->sub->sub  = (depl *) NULL;

  ip->sub->whiteturn =  mo->whiteturn;
  ip->sub->move = mo->move ;

  return(ip->sub);
}

/* Rewritten by Ed Maillet */
#ifdef __STDC__
void free_move_list(depl *d)
#else
void free_move_list(d)
     depl * d;
#endif
{
  depl *temp;
  for(;d!=NULL;temp=d,d=d->next,free(temp))
       if(d->sub!=NULL)
          free_move_list(d->sub);

}

/* procedure update board g with move m */
#ifdef __STDC__
void do_move(game *g,depl *m)
#else
void do_move(g,m)
     game *g;
     depl *m;
#endif
{
  switch (m->type) {
  case MOVE:
    setboard(g,m->tolig,m->tocol,m->piece,CURCOLOR(m)) ;
    clsboard(g,m->fromlig,m->fromcol) ;
    break;
  case PRISE:
    setboard(g,m->tolig,m->tocol,m->piece,CURCOLOR(m)) ;
    clsboard(g,m->fromlig,m->fromcol);
    break;
  case GRANDROQUE:
    if (m->whiteturn)
      m->fromlig = 1;
    else 
      m->fromlig = 8;
    setboard(g,m->fromlig,3,KING,CURCOLOR(m)) ;
    setboard(g,m->fromlig,4,ROOK,CURCOLOR(m)) ;
    clsboard(g,m->fromlig,1) ;
    clsboard(g,m->fromlig,5) ;
    break;
  case PETITROQUE:
    if (m->whiteturn)
      m->fromlig = 1;
    else 
      m->fromlig = 8;
    setboard(g,m->fromlig,7,KING,CURCOLOR(m)) ;
    setboard(g,m->fromlig,6,ROOK,CURCOLOR(m)) ;
    clsboard(g,m->fromlig,5) ;
    clsboard(g,m->fromlig,8) ;
    break;
  case EN_PASSANT:
    setboard(g,m->tolig,m->tocol,m->piece,CURCOLOR(m)) ;
    clsboard(g,m->fromlig,m->tocol) ;
    clsboard(g,m->fromlig,m->fromcol) ;
    break;
  case PROMOTION:
    setboard(g,m->tolig,m->tocol,m->promotion,CURCOLOR(m)) ;
    clsboard(g,m->fromlig,m->fromcol);
    break;
  case PROM_ET_PRISE:
    setboard(g,m->tolig,m->tocol,m->promotion,CURCOLOR(m)) ;
    clsboard(g,m->fromlig,m->fromcol);
    break;
  default:
    fprintf(stderr,"\nUnable to do move: unknown move type\n");
    break;
  }
}



/* this procedure undo the effect of move m on the board g */
#ifdef __STDC__
void undo_move(game *g,depl *m)
#else
void undo_move(g,m)
     game *g;
     depl *m;
#endif
{
  switch (m->type) {
  case MOVE:
    clsboard(g,m->tolig,m->tocol) ;
    setboard(g,m->fromlig,m->fromcol,m->piece,CURCOLOR(m)) ;
    break;
  case PRISE:
    setboard(g,m->tolig,m->tocol,m->prise,OPPCOLOR(m)) ;
    setboard(g,m->fromlig,m->fromcol,m->piece,CURCOLOR(m)) ;
    break;
  case GRANDROQUE:
    if (m->whiteturn)
      m->fromlig = 1;
    else 
      m->fromlig = 8;
    clsboard(g,m->fromlig,3) ;
    clsboard(g,m->fromlig,4) ;
    setboard(g,m->fromlig,5,KING,CURCOLOR(m)) ;
    setboard(g,m->fromlig,1,ROOK,CURCOLOR(m)) ;
    break;
  case PETITROQUE:
    if (m->whiteturn)
      m->fromlig = 1;
    else 
      m->fromlig = 8;
    clsboard(g,m->fromlig,6) ;
    clsboard(g,m->fromlig,7) ;
    setboard(g,m->fromlig,5,KING,CURCOLOR(m)) ;
    setboard(g,m->fromlig,8,ROOK,CURCOLOR(m)) ;
    break;
  case EN_PASSANT:
    clsboard(g,m->tolig,m->tocol) ;
    setboard(g,m->fromlig,m->tocol,PAWN,OPPCOLOR(m)) ;
    setboard(g,m->fromlig,m->fromcol,m->piece,CURCOLOR(m)) ;
    break;
  case PROMOTION:
    clsboard(g,m->tolig,m->tocol);
    setboard(g,m->fromlig,m->fromcol,PAWN,CURCOLOR(m)) ;
    break;
  case PROM_ET_PRISE:
    setboard(g,m->tolig,m->tocol,m->prise,OPPCOLOR(m)) ;
    setboard(g,m->fromlig,m->fromcol,PAWN,CURCOLOR(m)) ;
    break;
  default:
    fprintf(stderr,"\nUnable to undo move: unknown move type\n");
    break;
  }
}

