/*
  ChessPrn program by Ed Maillet
  @(#)movegen.c	1.0 Dated 3/93
  Copyright (C) 1993  Ed Maillet
  Modified and based on Notation 4.7 by Henry Thomas

 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "config.h"
#include "chesstyp.h"
#include "notation.h"

#include "lexer.h"
#include "movedo.h"
#include "movegen.h"


/* ------------------------------------------------------------ */

/* movement tables */
/* move only */
/* white pawn, move */
#define NB_M_PAWN_MOVE_WD 2
static int m_pawn_move_wd [][2] = {
  { 1, 0}, {2, 0}
};

/* black pawn, move */
#define NB_M_PAWN_MOVE_BD 2
static int m_pawn_move_bd [][2] = {
  {-1, 0}, {-2, 0}
};

/* TRICK = we have added the catching move at the end of
   the non catching ones; so in check_depl, we try first
   the non catching one and then the catching one.
   So, even if catching (x) is non indicated in the input,
   we succeed in guessing the move
   */
/* white pawn, move */
/*#define NB_M_PAWN_WD 2*/
#define NB_M_PAWN_WD 4
static int m_pawn_wd [][2] = {
  { 1, 0}, {2, 0},
/* catch... */
  { 1, 1}, { 1,-1}
};

/* white pawn, catch */
#define NB_M_PAWN_WX 2
static int m_pawn_wx [][2] = {
  { 1, 1}, { 1,-1}
};

/* black pawn, move */
/*#define NB_M_PAWN_BD 2*/
#define NB_M_PAWN_BD 4
static int m_pawn_bd [][2] = {
  {-1, 0}, {-2, 0},
/* catch... */
  {-1, 1}, {-1,-1} 
};

/* black pawn, catch */
#define NB_M_PAWN_BX 2
static int m_pawn_bx [][2] = {
  {-1, 1}, {-1,-1} 
};


#define NB_M_KNIGHT  8
static int m_knight[][2] = {
  { 2, 1}, { 2,-1}, {-2, 1}, {-2,-1},
  { 1, 2}, { 1,-2}, {-1, 2}, {-1,-2}
};

#define NB_M_BISHOP 28
static int m_bishop[][2] = {
  { 7, 7},  {6, 6}, { 5, 5}, { 4, 4}, { 3, 3}, { 2, 2}, { 1, 1},
  { 7,-7}, { 6,-6}, { 5,-5}, { 4,-4}, { 3,-3}, { 2,-2}, { 1,-1},
  {-7,-7}, {-6,-6}, {-5,-5}, {-4,-4}, {-3,-3}, {-2,-2}, {-1,-1},
  {-7, 7}, {-6, 6}, {-5, 5}, {-4, 4}, {-3, 3}, {-2, 2}, {-1, 1}
};

#define NB_M_ROOK 28
static int m_rook[][2] = {
  { 7, 0}, { 6, 0}, { 5, 0}, { 4, 0}, { 3, 0}, { 2, 0}, { 1, 0},
  {-7, 0}, {-6, 0}, {-5, 0}, {-4, 0}, {-3, 0}, {-2, 0}, {-1, 0},
  { 0, 7}, { 0, 6}, { 0, 5}, { 0, 4}, { 0, 3}, { 0, 2}, { 0, 1},
  { 0,-7}, { 0,-6}, { 0,-5}, { 0,-4}, { 0,-3}, { 0,-2}, { 0,-1}
};

#define NB_M_QUEEN 56
static int m_queen[][2] = {
  { 7, 7},  {6, 6}, { 5, 5}, { 4, 4}, { 3, 3}, { 2, 2}, { 1, 1},
  { 7,-7}, { 6,-6}, { 5,-5}, { 4,-4}, { 3,-3}, { 2,-2}, { 1,-1},
  {-7,-7}, {-6,-6}, {-5,-5}, {-4,-4}, {-3,-3}, {-2,-2}, {-1,-1},
  {-7, 7}, {-6, 6}, {-5, 5}, {-4, 4}, {-3, 3}, {-2, 2}, {-1, 1},
  { 7, 0}, { 6, 0}, { 5, 0}, { 4, 0}, { 3, 0}, { 2, 0}, { 1, 0},
  {-7, 0}, {-6, 0}, {-5, 0}, {-4, 0}, {-3, 0}, {-2, 0}, {-1, 0},
  { 0, 7}, { 0, 6}, { 0, 5}, { 0, 4}, { 0, 3}, { 0, 2}, { 0, 1},
  { 0,-7}, { 0,-6}, { 0,-5}, { 0,-4}, { 0,-3}, { 0,-2}, { 0,-1}
};

#define NB_M_KING 8
static int m_king[][2] = {
  { 1, 1}, { 1, 0}, { 1,-1},
  {-1, 1}, {-1, 0}, {-1,-1},
  { 0, 1}, { 0, -1}
};


/* table of alternate moves, guessed by
   the "move generator": guess depl
   */
static int alternate_moves[10][2];


/* ----------- semantic evaluation of move ----------- */
/* check if  position lies within the board
   */
#ifdef __STDC__
int in_board(int l,int c)
#else
int in_board(l,c)
     int l,c;
#endif
{
  return ((c >= 1) && (c <= 8) && (l >= 1) && (l <= 8));
}

/* check that the path from pos1 to pos2 is free
   but not including pos1 or pos2!!

   Return:
     (int) TRUE if path is free; FALSE otherwise
*/
#ifdef __STDC__
int path_free(game *tos,int l1,int c1,int l2,int c2)
#else
int path_free(l1, c1, l2, c2)
game *tos;
int l1,c1, l2, c2;
#endif
{
  int li = 1 ;
  int ci = 1 ;
  int lig, col;


  li = SIGN(l2-l1);
  ci = SIGN(c2-c1);


  if ( c1 == c2 )
  {
    col = c1;
    for (lig = l1 +li; lig != l2 ; lig +=li)
      if (tos->board[lig][col] != VOID)
	return (FALSE);
    return(TRUE);
  }

  if ( l1 == l2)
  {
    lig = l1 ;
    for (col = c1 + ci; col != c2 ; col +=ci)
      if (tos->board[lig][col] != VOID)
	return (FALSE);
    return(TRUE);
  }

  for (lig = l1+li,col =c1+ci; (lig!=l2) && (col!=c2); lig+=li, col+= ci)
    if (tos->board[lig][col] != VOID)
        return (FALSE);
  return(TRUE);
}

/* check if roque is possible. King can't be under attack and squares
   king passes over can not be attacked
   The check for the king having been moved has not been done.

   Rewritten by Ed Maillet
*/
#ifdef __STDC__
int check_roque(game *tos,depl *m)
#else
int check_roque()
    game *tos;
    depl *m;
#endif
{
  int lig, col ;

  if(KingAttacked(tos,m->whiteturn==TRUE?WHITE:BLACK))
     return FALSE;

  if (m->whiteturn)
    lig = 1 ;
  else
    lig =8;
  if (m->type == GRANDROQUE)
    {
     if(tos->board[lig][1]!=ROOK || tos->color[lig][1]!=CURCOLOR(m))
        return(FALSE);
     for (col = 3; col < 5 ; col++)
      if (tos->board[lig][col] != VOID ||
          SquareAttacked(tos,lig,col,OPPCOLOR(m)))
	return(FALSE);
    }
  if (m->type == PETITROQUE)
   {
    if(tos->board[lig][8]!=ROOK || tos->color[lig][8]!=CURCOLOR(m))
       return(FALSE);
    for (col = 6; col < 7 ; col++)
      if (tos->board[lig][col] != VOID ||
          SquareAttacked(tos,lig,col,OPPCOLOR(m)))
	return(FALSE);
   }
  return(TRUE);
}


#ifdef __STDC__
int check_numbering(depl *m , int n)
#else
int check_numbering(m, n)
     depl * m;
     int n;
#endif
{
  int real_n;

  real_n = m->move ;
  /* we have to adjust the move number, because m->move
     refers to the previous move
     */
  if (m->whiteturn == FALSE) /* last move was a black one */
    real_n++;
  if (n != real_n)
   {
/*    message((stderr,"\n%s, line %d: move number should be %d\n",
	     filename, lineno, real_n));
*/
    return(FALSE);
   } else
    return(TRUE);
}


/* check -- or guess -- where a given piece come */
#ifdef __STDC__
int guess_piece(void)
#else
int guess_piece()
#endif
{
  return(tos->board[m->fromlig][m->fromcol]);
}

/* try to guess the move -- low-level function
   it returns -- in the parms -- the coordinates of a possible move
   it returns -- as value -- the number of possible moves
   Rewritten by Ed Maillet
 */
#ifdef __STDC__
int guess_depl(int nb, int tab[][2],
	       int * pl1, int * pc1, int l2, int c2)
#else
int guess_depl(nb, tab, pl1, pc1, l2,c2
     int nb;
     int tab[10][2];
     int *pl1, *pc1;
     int l2,c2;
#endif
{
  int i,l,c;
  int count = 0;
  int tBoard[SIZE][SIZE]; /* Used for fast look up of already found moves */
  int rank[SIZE];         /* ranks of enemy pieces in a file */
  int rankindx,numPawns,dir;

  memset((void *)tBoard,0,SIZE*SIZE*sizeof(int));

  numPawns=1;
  dir=OPPCOLOR(m)==BLACK?1:-1; /* direction of pawn motion in board array */
  if(l2==0) /* moves like "cd" the tolig isn't known */
    for(i=1;i<9;++i)
      {
       if(tos->color[i][c2]==OPPCOLOR(m))
          rank[numPawns++]=i;
       if(tos->board[i][c2]==PAWN && /* "cd" could be an ep move */
          tos->board[i+dir][c2]==VOID)
            rank[numPawns++]=i+dir;
      }

 for(rankindx=1;rankindx<=numPawns;++rankindx)
  {
   if(numPawns>1)
      l2=rank[rankindx];
   for (i=0; i< nb; i++ )
    {
     l = l2 - tab[i][0];
     c = c2 - tab[i][1];
     if(LegalMove(tos,m,l,c,l2,c2)) /* Ed Maillet */
       if(!tBoard[l][c]) /* move already found? */
        {
         alternate_moves[count][0] = l;
         alternate_moves[count][1] = c;
         count++;
         tBoard[l][c]=TRUE;
         if(numPawns>1)
           {
             m->tocol=c2;
             m->tolig=l2;
           }
        }
    } /* for i */
  } /* for rankindx */
  alternate_moves[count][0] = alternate_moves[count][1] = 0;
  if (count==1)
  { *pl1 = alternate_moves[0][0]; *pc1 = alternate_moves[0][1]; }
  else if (count>1)
    count=ambiguity(count,m,pc1,pl1);
  m->prise=tos->board[m->tolig][m->tocol];
  if(tos->board[m->tolig][m->tocol]!=VOID)
     m->type=PRISE;
  if(m->type!=PROMOTION && m->type!=PROM_ET_PRISE &&
     m->piece == PAWN && (m->tolig==1 || m->tolig==8))
    { m->promotion=QUEEN;
      m->type=(m->type==PRISE)?PROM_ET_PRISE:PROMOTION;
    }

  return(count);
}

/* ambiguity - resolve ambiguity in a move
   Given an ambiguous move try to resolve it using the
   fromlig and fromcol fields in the depl. The ambrank and
   ambfile flags are also set.

   Parameters:
     int count - number of potential moves
     depl *m   - current move node
     int *pc1  - potential from file to be set
     int *pl1  - potential from rank to be set

   Return:
     (int) the number of potential moves still available after using
           the fromlig and fromcol fields
           == 1 if ambiguity resolved; unresolved otherwise

 Rewritten by Ed Maillet
   */
#ifdef __STDC__
int ambiguity(int count,depl *m,int *pc1,int *pl1)
#else
int ambiguity(count,m,pc1,pl1)
     int count;
     depl *m;
     int *pc1;
     int *pl1;
#endif
{
 int i,movecnt;

  *pc1=0;
  *pl1=0;
  m->ambrank = TRUE ;
  m->ambfile = TRUE ;
  for (i= 0; i < count ; i++)
       {
        if (alternate_moves[i][0] != alternate_moves[0][0])
             m->ambfile = FALSE;
        if (alternate_moves[i][1] != alternate_moves[0][1])
             m->ambrank = FALSE;
       }
  /* R's on E4 & D1 with move Rd4 gets missed by above test */
  if(m->ambrank==FALSE && m->ambfile==FALSE)
     m->ambfile = TRUE;

  /* try to resolve ambiguity */
  if(m->fromcol!=0 && m->fromlig!=0)
   for(movecnt=0,i=0;i<count;++i)
   {
    if(m->fromlig==alternate_moves[i][0] && m->fromcol==alternate_moves[i][1])
      {
       *pl1 = alternate_moves[i][0];
       *pc1 = alternate_moves[i][1];
       ++movecnt;
      }/* if */
    } /* for */
    else if(m->fromlig!=0)
     for(movecnt=0,i=0;i<count;++i)
      {if(m->fromlig==alternate_moves[i][0])
        {
         *pl1 = alternate_moves[i][0];
         *pc1 = alternate_moves[i][1];
         ++movecnt;
        }/* if */
      }/* for */
   else if(m->fromcol!=0)
    for(movecnt=0,i=0;i<count;++i)
    {
     if(m->fromcol==alternate_moves[i][1])
       {
        *pl1 = alternate_moves[i][0];
        *pc1 = alternate_moves[i][1];
        ++movecnt;
       } /* if */
    }/* for */
   else /* no from col given */
     ++movecnt;
  if(movecnt!=1) { *pc1=0; *pl1=0;}
  return movecnt;
}

/* try to guess the move -- used for shortened notation
   */
#ifdef __STDC__
int guess_move(depl *m)
#else
int guess_move()
    depl *m;
#endif
{
  int l1,c1,l2,c2;
  int r;
  char *colorStr;

  if ((m->type == GRANDROQUE) || (m->type == PETITROQUE))
       return(check_roque(tos,m));

  l1 = m->fromlig ;
  c1 = m->fromcol ;
  l2 = m->tolig;
  c2 = m->tocol;

  switch(m->piece) {
  case PAWN:
    if (m->type==PRISE)
     {
      if (m->whiteturn)
	  r=guess_depl(NB_M_PAWN_WX, m_pawn_wx, &l1,&c1, l2,c2);
      else
	  r=guess_depl(NB_M_PAWN_BX, m_pawn_bx, &l1,&c1, l2,c2);
    } else {
      if (m->whiteturn)
	  r=guess_depl(NB_M_PAWN_WD, m_pawn_wd, &l1,&c1, l2,c2);
      else
	  r=guess_depl(NB_M_PAWN_BD, m_pawn_bd, &l1,&c1, l2,c2);
    }
    break;
  case KNIGHT:
       r=guess_depl(NB_M_KNIGHT, m_knight, &l1,&c1, l2,c2);
    break;
  case BISHOP:
       r=guess_depl(NB_M_BISHOP, m_bishop, &l1,&c1, l2,c2);
    break;
  case ROOK:
       r=guess_depl(NB_M_ROOK, m_rook, &l1,&c1, l2,c2);
    break;
  case QUEEN:
       r=guess_depl(NB_M_QUEEN, m_queen, &l1,&c1, l2,c2);
    break;
  case KING:
       r=guess_depl(NB_M_KING, m_king, &l1,&c1, l2,c2);
    break;
  default:
    r=0; /* Invalid piece */
    break;
  }

  if (((l1 == 0) || (c1 == 0)) && r!=0)
   { colorStr=(m->whiteturn)?"white":"black";
     error((stderr,"\n%s, line %d: Ambiguous %s move %d, with piece %c\n",
           filename, lineno, colorStr,m->move,in_table[m->piece]));
   }
   else if (r==0)
    { colorStr=(m->whiteturn)?"White":"Black";
     error((stderr,"\n%s, line %d: %s move %d illegal: ",filename,lineno,colorStr,m->move));
    }
   else
   {m->fromcol = c1;
    m->fromlig = l1;
    return TRUE;
   }
  return FALSE;
}
/* Check - Check to see if removing piece on l2 c2 and putting it on
   l1 c1 places my king in check.

   Parameters:
     game *g - pointer to current board
     depl *m - pointer to move info
     int l2,c2 - piece to be removed location
     int l1,c1 - location of piece destination
   Return:
    (int) TRUE if removing piece puts king in check
          FALSE otherwise

 Written by Ed Maillet
*/
#ifdef __STDC__
int Check(game *g,depl *m,int l2,int c2,int l1,int c1)
#else
int Check(g,m,l2,c2,l1,c1)
    game *g;
    depl *m;
    int l2;
    int c2;
    int l1;
    int c1;
#endif
{
  int color,flag;
  game tgame;
  struct deplace move;

  color = CURCOLOR(m);
  tgame= *g; /* Paranoid and proud! */
  /* remove piece */
  move = *m;
  move.tolig=l1;
  move.tocol=c1;
  move.fromlig=l2;
  move.fromcol=c2;
  do_move(g,&move);
  flag=KingAttacked(g,color);
  undo_move(g,&move);
  *g=tgame;
  return flag;
}

/* KingAttacked - Check to see if king of given color
   is attacked on given board

   Parameters:
     game *g - board to be tested
     int color - color of king to check
   Return:
    (int) TRUE if king is in check; FALSE otherwise

   Written by Ed Maillet
*/
#ifdef __STDC__
int KingAttacked(game *g,int color)
#else
int KingAttacked(g,color)
    game *g;
    int color;
#endif
{
 int flag;
 int i,j,l2,c2;
  /* find king */
  for(i=1;i<9;++i)
   for(j=1;j<9;++j)
       if(g->board[i][j]==KING && g->color[i][j]==color)
          { l2=i; c2=j; i=9; j=9; break; }

   flag= SquareAttacked(g,l2,c2,color==WHITE?BLACK:WHITE);

   return flag;
}

/* LegalMove - Check if proposed move is legal

   Parameters:
     game *g - current board position
     depl *m - partial move (piece and destination)
     int l1,c1 - source location of piece proposed to move
     int l2,c2 - destination of piece

   Return:
    FALSE if move illegal
    TRUE otherwise.

  Written by Ed Maillet
*/
#ifdef __STDC__
int LegalMove(game *tos,depl *m,int l1,int c1,int l2,int c2)
#else
int LegalMove(tos,m,l1,c1)
     game *tos;
     depl * m;
     int l1;
     int c1;
     int l2;
     int c2;
#endif
{
  int capture=FALSE; /* TRUE if proposed destination is not empty */

    if (!in_board(l1,c1) || !in_board(l2,c2))
        return FALSE;

    if((tos->board[l1][c1] != m->piece) ||
       (tos->color[l1][c1] != CURCOLOR(m)))
         return FALSE;

    if(!path_free(tos,l1,c1,l2,c2))
       return FALSE;

  if ((m->type == GRANDROQUE) || (m->type == PETITROQUE))
    return(check_roque(tos,m));

  if (tos->board[l2][c2] != VOID)
      capture=TRUE;

  /* Attempt to move onto my own piece */
  if (tos->color[l2][c2] == tos->color[l1][c1])
      return FALSE;

  switch(m->piece)
    {
     case PAWN:
            /* check en passant */
          if(!capture && tos->board[l1][c2]==PAWN &&
             tos->color[l1][c2]==OPPCOLOR(m) && m->prev!=NULL &&
             m->prev->piece==PAWN && m->prev->fromlig!=l2 &&
             m->prev->fromcol==c2)
              { m->type=EN_PASSANT; return TRUE; }
          if (capture && (c1==c2))/* Pawns can't capture forward */
               return FALSE;
          if(!capture && (c1!=c2))
               return FALSE;
          break;
     default:
          break;
    }
    if(Check(tos,m,l1,c1,l2,c2)) /* Ed Maillet */
       return FALSE;

  return TRUE;
}


/* SquareAttacked - Check if square is attacked given a board position

   Parameters:
      game *g - pointer to board position
      int row - square row
      int col - square col
      int color - color of ATTACKING pieces
   Returns:
      (int) TRUE if square row,col is attacked, FALSE otherwise

   Written by Ed Maillet
*/
#ifdef __STDC__
int SquareAttacked(game *g,int row,int col,int color)
#else
int SquareAttacked(g,row,col,color)
      game *g;
      int row;
      int col;
      int color;
#endif
{
  int limit[]={NB_M_BISHOP,NB_M_ROOK,NB_M_KNIGHT,NB_M_KING};
  int *moves[]={&m_bishop[0][0],&m_rook[0][0],&m_knight[0][0],&m_king[0][0]};
  int pieces[][2]={BISHOP,QUEEN,ROOK,QUEEN,KNIGHT,KNIGHT,KING,KING};
  int plimit[]={NB_M_PAWN_BX,NB_M_PAWN_WX};
  int *pmoves[]={&m_pawn_bx[0][0],&m_pawn_wx[0][0]};
  int i,j,l,c,index;


  /* Major Pieces */
  for(j=0;j<4;++j)
   for(i=0;i<limit[j];++i)
      {
       l=row - *(moves[j]+(i*2)); /* Yipes! */
       c=col -  *(moves[j]+(i*2)+1);
       if(in_board(l,c) && g->color[l][c]==color &&
         (g->board[l][c]==pieces[j][0] || g->board[l][c]==pieces[j][1]) &&
         path_free(g,l,c,row,col))
         return TRUE;
      }
  /* Pawns */
   index=(color!=BLACK);
   for(i=0;i<plimit[index];++i)
      {
       l=row- *(pmoves[index]+(i*2));
       c=col- *(pmoves[index]+(i*2)+1);
       if(in_board(l,c) && g->color[l][c]==color && g->board[l][c]==PAWN)
         return TRUE;
      }

  return FALSE;
}

/* IsMate -- Check if king of given color is mated on given board

   Parameters:
      game *tos - board to be checked
      int color - color of king to be checked
   Retrurn:
     (int) TRUE if King is Mated; FALSE otherwise

  Written by Ed Maillet
*/
#ifdef __STDC__
int IsMate(game *tos,int color)
#else
int IsMate(tos,color)
     game *tos;
     int color;
#endif
{
  if(!KingAttacked(tos,color))
     return FALSE;
  if(AnyLegalMoves(tos,color))
     return FALSE;
  return TRUE;
}

/* AnyLegalMoves - Check if there are any legal moves for given
   color on given board. This function returns as soon as 1 legal
   move is found. It can be used for testing for mate and
   stalemate.

   Parameters:
      game *tos - board to be checked
      int color - color of moving pieces
   Return:
     (int) TRUE if at least 1 legal move exists; FALSE otherwise

   Written by Ed Maillet
*/

#ifdef __STDC__
int AnyLegalMoves(game *tos,int color)
#else
int AnyLegalMoves(tos,color)
     game *tos;
     int color;
#endif
{
 int rank,file,i,row,col,retval=FALSE;
 struct deplace fake;
 memset((void *)&fake,0,sizeof(fake));
 fake.whiteturn=color==WHITE;
 for(rank=1;rank<9;++rank)
  for(file=1;file<9;++file)
     if(tos->color[rank][file]==color &&
        tos->board[rank][file]!=VOID)
       switch(tos->board[rank][file])
        {
         case QUEEN:
              fake.piece=QUEEN;
              for(i=0;i<NB_M_QUEEN;++i)
                 {row=m_queen[i][0]; col=m_queen[i][1];
                  if(tos->board[row][col]!=VOID)
                   { fake.type=PRISE; }
                  else
                   { fake.type=MOVE; }
                  if(LegalMove(tos,&fake,rank,file,rank+row,file+col))
                    { retval=TRUE; break; }
                 }
              break;
         case ROOK:
              fake.piece=ROOK;
              for(i=0;i<NB_M_ROOK;++i)
                 {row=m_rook[i][0]; col=m_rook[i][1];
                  if(tos->board[row][col]!=VOID)
                   { fake.type=PRISE; }
                  else
                   { fake.type=MOVE; }

                  if(LegalMove(tos,&fake,rank,file,rank+row,file+col))
                    { retval=TRUE; break; }
                 }
              break;
         case KNIGHT:
              fake.piece=KNIGHT;
              for(i=0;i<NB_M_KNIGHT;++i)
                 {row=m_knight[i][0]; col=m_knight[i][1];
                  if(tos->board[row][col]!=VOID)
                   { fake.type=PRISE; }
                  else
                   { fake.type=MOVE; }

                  if(LegalMove(tos,&fake,rank,file,rank+row,file+col))
                    { retval=TRUE; break; }
                 }
              break;
         case BISHOP:
              fake.piece=BISHOP;
              for(i=0;i<NB_M_BISHOP;++i)
                 {row=m_bishop[i][0]; col=m_bishop[i][1];
                  if(tos->board[row][col]!=VOID)
                   { fake.type=PRISE; }
                  else
                   { fake.type=MOVE; }

                  if(LegalMove(tos,&fake,rank,file,rank+row,file+col))
                    { retval=TRUE; break; }
                 }
              break;
         case KING:
              fake.piece=KING;
              fake.type=GRANDROQUE;
              if(check_roque(tos,&fake))
                 { retval=TRUE; break;}
              fake.type=PETITROQUE;
              if(check_roque(tos,&fake))
                 {retval=TRUE; break; }
              for(i=0;i<NB_M_KING;++i)
                 {row=m_king[i][0]; col=m_king[i][1];
                  if(tos->board[row][col]!=VOID)
                   { fake.type=PRISE; }
                  else
                   { fake.type=MOVE; }

                  if(LegalMove(tos,&fake,rank,file,rank+row,file+col))
                    { retval=TRUE; break; }
                 }
              break;
         case PAWN: /* no ep test! */
             fake.piece=PAWN;
             if(color==BLACK)
               {
                for(i=0;i<NB_M_PAWN_BD;++i)
                  { row=m_pawn_bd[i][0]; col=m_pawn_bd[i][1];
                    if(tos->board[row][col]!=VOID)
                      { fake.type=PRISE; }
                     else
                      { fake.type=MOVE; }
                    if(LegalMove(tos,&fake,rank,file,rank+row,file+col))
                      { retval=TRUE; break; }
                  }
               }
              else
               {
                for(i=0;i<NB_M_PAWN_WD;++i)
                  { row=m_pawn_wd[i][0]; col=m_pawn_wd[i][1];
                    if(tos->board[row][col]!=VOID)
                      { fake.type=PRISE; }
                     else
                      { fake.type=MOVE; }
                    if(LegalMove(tos,&fake,rank,file,rank+row,file+col))
                       { retval=TRUE; break; }
                  }
               }
              break;
        } /* switch */
 return retval;
}

