/*
  ChessPrn program
  @(#)notation.c 1.0 Dated 3/93
  Copyright (C) 1993  Ed Maillet
  Modified and based on Notation 4.7 by Henry Thomas
 */
/*
This file is part of ChessPrn program.

ChessPrn is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 1, or (at your option)
any later version.

ChessPrn is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ChessPrn see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

#ifdef __STDC__
#include <stdlib.h>
#endif
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "config.h"
#include "chesstyp.h"
#include "notation.h"
#include "movedo.h"
#include "movegen.h"
#include "drivers.h"
#include "lexer.h"

/* --------------------- data part ---------------------- */

/* les tableaux suivants sont les tables de transcription de notation
   selon les langages
   */

extern void close_files();

char * version_string =
  "@(#)ChessPrn 1.0 Dated 3/93\n";


static char * keywords[]= {
  "@startplay" , "@clearboard" , "@showboard" ,
  "@whitesmove", "@blacksmove", "@configwhite", "@configblack" ,
  "@default" ,
  /* these are keywords with arguments */
  "@title", "@subtitle", "@language",
  /* "special" keyword */
  "@special",
  /* null and final keyword */
  "@null"
  };

int configuring = FALSE ;
int configside = 0 ;


static char * t_language[] = {
  "french", "english", "italian", "spanish", "german", "dutch",
  "czech",  "hungarian","polish", "romanian", "FIDE","danish",
  "estonian","finnish","norwegian","portuguese","swedish"
};

static int in_language = IN_LANG ;
static int out_language = OUT_LANG ;

static char c_language[NBLANGUAGES][7] = {
/* french    */  { '@' ,'R' , 'D' , 'T' , 'F' , 'C' , 'P' },
/* english   */  { '@' ,'K' , 'Q' , 'R' , 'B' , 'N' , 'P' },
/* italian   */  { '@' ,'R' , 'D' , 'T' , 'A' , 'C' , 'P' },
/* spanish   */  { '@' ,'R' , 'D' , 'T' , 'A' , 'C' , 'P' },
/* german    */  { '@' ,'K' , 'D' , 'T' , 'L' , 'S' , 'B' },
/* dutch     */  { '@' ,'K' , 'D' , 'T' , 'L' , 'P' , 'O' },
/* czech     */  { '@' ,'K' , 'D' , 'V' , 'S' , 'J' , 'P' },
/* hungarian */  { '@' ,'K' , 'V' , 'B' , 'F' , 'H' , 'G' },
/* polish    */  { '@' ,'K' , 'H' , 'W' , 'G' , 'S' , 'P' },
/* romanian  */  { '@' ,'R' , 'D' , 'T' , 'N' , 'C' , 'P' },
/* FIDE      */  { '@' ,'K' , 'D' , 'T' , 'S' , 'N' , 'P' },
/* Danish    */  { '@' ,'K' , 'D' , 'T' , 'L' , 'S' , 'P' },/* P assumed */
/* Estonian  */  { '@' ,'K' , 'L' , 'V' , 'O' , 'R' , 'P' },/* P assumed */
/* Finnish   */  { '@' ,'K' , 'D' , 'T' , 'L' , 'R' , 'P' },/* P assumed */
/* Norwegian */  { '@' ,'K' , 'D' , 'T' , 'L' , 'S' , 'P' },/* P assumed */
/* Portuguese*/  { '@' ,'R' , 'D' , 'T' , 'L' , 'S' , 'P' },/* P assumed */
/* Swedish   */  { '@' ,'K' , 'D' , 'T' , 'L' , 'S' , 'P' } /* P assumed */
/* UNIMPLEMENTED ... */
/* user_def  *//*{ '@' ,'X' , 'X' , 'X' , 'X' , 'X' , 'X' }*/
/* russian not implemented : ASCII russian is an oxymoron */
/* russian   *//*{ '@' ,'K' , 'F' , 'D' , 'C' , 'K' , 'P' }*/
	       };

/* input translation table */
char *in_table;

char *  c_roque[] = { "O-O" , "O-O-O" , "o-o" , "o-o-o" , "0-0" , "0-0-0" };

/* various notations for en passant */
#define N_EP 2
char * c_en_passant[] = { "ep" , "e.p." } ;


/* notation for catch */
char c_prise ='x';

/* various comments */
char * c_comments[] = { "+" , "++" ,
			  "?" , "??", "!", "!!", "!?", "?!",
			  "mate", "draw" };


/* I/O */
FILE * infile ;
FILE * fhelp;

static char * t_output[] =
{ "ascii", "postscript", "tex", "roff", "xchess", "gnu",
  "zarkov","cm","pgn","chessarc","ps"};


/* the stack itself */
stack_elt stack[VARIATION_MAX];

/* top of the stack */
/* --> explicit in dr->variation */


/* ---------- automata definitions --------- */
/* table for syntaxic analysis of move */

#define FINAL	10
#define TML 	FINAL   /* terminal state */
#define NBETAT 	13
#define NBCLAS 	9 /* Ed Maillet */

/* successor of state */
static int transit[NBETAT][NBCLAS] = {
/*   P a-h 1-8   -   x   =  \0   ?  :\ */
/*(  0   1   2   3   4   5   6   7   8)*/
  {  1, 11, -1, -1, -1, -1, -1, -1,  0 }, /* etat  0 */
  { -1,  2,  3, -1,  4, -1, -1, -1,  1 }, /* etat  1 */
  { -1,  6,  3,  4,  4,  8,TML,TML,  2 }, /* etat  2 */
  { -1,  6, -1,  4,  4,  8,TML,TML,  3 }, /* etat  3 */
  {  5,  6, -1, -1, -1, -1, -1, -1,  4 }, /* etat  4 */
  { -1,  6, -1, -1, -1, -1, -1, -1,  5 }, /* etat  5 */
  { -1, -1,  7, -1, -1, -1, -1, -1,  6 }, /* etat  6 */
  { -1, -1, -1, -1, -1,  8,TML,TML,  7 }, /* etat  7 */
  {  9, -1, -1, -1, -1, -1,TML, -1,  8 }, /* etat  8 */
  { -1, -1, -1, -1, -1, -1,TML,TML,  9 }, /* etat  9 */
  { -1, -1, -1, -1, -1, -1, -1, -1, 10 }, /* etat 10 == terminal */
  { -1, 12,  3, -1,  4, -1, -1, -1, 11 }, /* etat 11 - Ed Maillet */
  { -1, -1,  9, -1,  4, -1,TML, -1, 12 }  /* etat 12 - Ed Maillet */
};

/* actions to do */
static int action[NBETAT][NBCLAS] = {
/*   P a-h 1-8   -   x   =  \0   ?  :\ */
/*(  0   1   2   3   4   5   6   7   8)*/
  {  1,  2, -1, -1, -1, -1, -1, -1, -1}, /* etat  0 */
  { -1,  2,  3, -1, 10, -1, -1, -1, -1}, /* etat  1 */
  { -1, 13,  3,  4,  5, 14,  6,  7, -1}, /* etat  2 */
  { -1, 13, -1,  4,  5, 14,  6,  7, -1}, /* etat  3 */
  {  1,  2, -1, -1, -1, -1, -1, -1, -1}, /* etat  4 */
  { -1,  2, -1, -1, -1, -1, -1, -1, -1}, /* etat  5 */
  { -1, -1,  3, -1, -1, -1, -1, -1, -1}, /* etat  6 */
  { -1, -1, -1, -1, -1, 14,  8,  9, -1}, /* etat  7 */
  { 15, -1, -1, -1, -1, -1, 17, -1, -1}, /* etat  8 */
  { -1, -1, -1, -1, -1, -1, 17, 17, -1}, /* etat  9 */
  { -1, -1, -1, -1, -1, -1, -1, -1, -1}, /* etat 10 */
  { -1, 16,  3, -1,  5, -1, -1, -1, -1}, /* etat 11 - Ed Maillet */
  { -1, -1, 18, -1, -1, -1, 17, -1, -1}  /* etat 12 - Ed Maillet */
};


/* the complete play */
play * theplay ;

/* current game
   the name "tos" means "top of stack"
   */
game * tos = GULL ;

/* variable holding current move */
depl * m = MULL ;


/* the output driver */
format * dr;

static int driver; /* driver type, ie gnu, ascii ... */

static int movecount;
static int checkmovenumbers = TRUE ;

/* current move, used by the parser */
static int curpiece,  curcol,  curlig ;
static int curdigit, curmove;

/* booleen d'erreur */
int error_flag = FALSE;

static int move_to_display[NB_MOVE_TO_DISP] ;
static int nb_move_to_dsp = 0;
static int stop_at_display = FALSE;

/* short and long form comment table */
char * com_short[] = {
#define CHESSSYMB(LET,LASC,SASC,TEX,PS,ENG,FRA) SASC,
#include "chesssym.def"
 ""
 };
#undef CHESSSYMB

char * com_long[] = {
#define CHESSSYMB(LET,LASC,SASC,TEX,PS,ENG,FRA) LASC,
#include "chesssym.def"
 ""
 };
#undef CHESSSYMB


/* --------------------------- code part --------------------- */

/* GetLangChar - Get Language Character of piece
   Language and pices are not range checked.

   Parameters:
      int lang - language ENGLISH, FRENCH ....
      int piece - KING, QUEEN, ....
   Return:
      (char) char of piece in language lang
*/
#ifdef __STDC__
char GetLangChar(int lang,int piece)
#else
char GetLangChar(lang,piece)
     int lang;
     int piece;
#endif
{
  return c_language[lang][piece];
}



#ifdef __STDC__
static int ispiece(char c)
#else
static int ispiece(c)
     char c;
#endif
{
  register int i;
  
  for ( i = 0 ; (i < NUMPIECES) && (c != in_table[i]) ; i++ ) ;
  /*(void) fprintf(stdout, "piece %d %c\n" , i , c);*/
  return(i<NUMPIECES);
}


#ifdef __STDC__
static int piece(char c)
#else
static int piece(c)
     char c ;
#endif
{
  register int i;

  for ( i = 0 ; (i < NUMPIECES) && (c != in_table[i]) ; i++ ) ;
  if ( i== NUMPIECES)
    i = PAWN ;
  return(i);
}

/* this function returns the rank of a keyword in a given table.
   if key is not present, it returns the default value
   */
#ifdef __STDC__
static int find_keyword(char *tab[], int nbentry,int defaut,
			char *key,int warning)
#else
static int find_keyword(tab, nbentry,defaut,key,warning)
     char * tab[]; /* the table to look in */
     int nbentry;  /* number of entries */
     int defaut;  /* the default value to return if search failed */
     char *key;    /* the key to find */
     int warning;  /* do we display a warning ? */
#endif
{
  int i ;

  for(i=0; (i< nbentry) ;i++)
    if (strcmp(tab[i],key)==0)
      return(i);

  /* we failed to find the keyword */
  if (warning)
    (void) fprintf (stderr, "unknow keyword %s in this context\n",key);
  return(defaut);
}


/* --------------- execution of move ----------------- */

/* clear a position */
#ifdef __STDC__
int clear_pos(int lig, int col)
#else
int clear_pos(lig,col)
     int lig;
     int col;
#endif
{
  tos->board[lig][col] = VOID ;
  tos->color[lig][col] = VOID ;
  return(TRUE);
}

/* configure the board */
#ifdef __STDC__
int configure(void)
#else
int configure()
#endif
{
  if (configuring) {
    if (m->piece == VOID)
      m->piece = PAWN ;
    tos->board[m->tolig][m->tocol] = m->piece ;
    tos->color[m->tolig][m->tocol] = configside ;
  }
  return(TRUE);
}

/* variation procedures == stack manipulation */

#ifdef __STDC__
void enter_variation(void)
#else
void enter_variation()
#endif
{
  int l;

  l = dr->variation ;

  if (l >= VARIATION_MAX) {
    error((stderr,"\n%s, line %d: Maximum imbricated variation is %d\n",
	   VARIATION_MAX, filename, lineno));
  } else {
    /* save current line/variation */
    stack[l].d = m;
    stack[l].b = tos;
    stack[l].d1 = dr->iswhiteturn;
    stack[l].d2 = dr->intrupt = TRUE ;
    /* create new */
    tos = new_board();
    (void) copy_board(stack[l].b, tos);

    /* A variation FOLLOWS the main line
       so we need to backtrack one move
       */
    m = add_variation(stack[l].d);
    undo_move(tos,stack[l].d);

    /* set variables */
    l++;
    dr->variation = l;

    output_variation(dr,VARIATION_IN);
  }
}

#ifdef __STDC__
void exit_variation(void)
#else
void exit_variation()
#endif
{
  int l ;

  l = dr->variation ;

  if (l == 0) {
    error((stderr,"\n%s, line %d: You cannot exit from the main line (a variation level error?)\n", filename, lineno));
  } else {
    output_variation(dr,VARIATION_OUT);

    l--;
    free(tos);
    m = stack[l].d ;
    m->next=NULL;
    tos = stack[l].b ;

    dr->iswhiteturn = stack[l].d1 ;
    dr->intrupt = stack[l].d2 ;
    dr->variation = l;
  }
}

/* ------------------------------------------------------------ */
/* execute a move, no checking */
#ifdef __STDC__
int execute_move(void)
#else
int execute_move()
#endif
{
  register int i;
  game *backup;

  /* do we need to display the previous move ? */
  if (nb_move_to_dsp > 0)
    { for (i=0; i < nb_move_to_dsp; i++)
       if (m->move == (move_to_display[i]+1 ) && m->whiteturn )
       { output_board(dr,tos);
	 if (stop_at_display)
         { output_end(dr);
	   close_files();
	   exit(0);
	 }
       }
    }



  if (m->piece == VOID)
   if(m->fromlig!=0 && m->fromcol!=0)
     m->piece=guess_piece();
    else
    m->piece = PAWN;

  (void) guess_move(m);

  /* do a copy of the board, to keep a valid board
     if the move is false
     */

  backup = new_board();
  copy_board(tos, backup);

  do_move(tos, m);
  if(!error_flag && IsMate(tos,OPPCOLOR(m)))
     m->is_check=MATE;
  else if (!error_flag && KingAttacked(tos,OPPCOLOR(m)))
     m->is_check=KINGCHECK;
   else
     m->is_check=FALSE;

  output_move(dr,m);

  if (error_flag) {
    (void) fprintf(dr->outfile, "\nLast valid board is:");
    output_board(dr,backup);
    free(backup);
    close_files();
    exit(0);
  }

  free(backup);

  return(TRUE);
}

/* ------------------ automata ----------------------- */

/* categorise the input for the automata */
#ifdef __STDC__
int typechar(char c)
#else
int typechar(c)
     char c;
#endif
{
  if (ispiece(c))
    return(0);
  if ((c >=  'a') && ( c <= 'h'))
    return(1);
  if ((c >=  '1') && ( c <= '8'))
    return(2);
  if ( c== '-' )
    return(3);
  if ((c == 'x') || (c == 'X' ))
    return(4);
  if (c == '=' )
    return(5);
  if (c == '\0' )
    return(6);
  if(c==':' || c=='/')
    return 8;
  return(7);
}


/* execute the actions decided by the automata */
#ifdef __STDC__
int execute(int num,char c)
#else
int execute(num,c)
     int num;
     char c;
#endif
{
  switch (num) {
  case 1: /* set cur piece */
    curpiece = piece(c);
    break;
  case 2: /* set cur col */
    curcol = lettertocol(c);
    break;
  case 3: /* set cur lig */
    curlig = lettertolig(c);
    break;
  case 4: /* from = cur ; prise = false */
    m->piece = curpiece ;
    m->fromcol = curcol ;
    m->fromlig = curlig;
    /*m->topiece = curpiece;*/
    break;
  case 5: /* from = cur ; prise = true */
    m->piece = curpiece ;
    m->fromcol = curcol ;
    m->fromlig = curlig;
    m->type = PRISE ;
    m->prise = curpiece;
    break;
  case 6: /* to = cur ; guess from */
  case 7: /* to = cur ; guess from ; parse remaining token */
    m->piece = curpiece ;
    m->tocol = curcol;
    m->tolig = curlig ;

    /*m->topiece = curpiece ; /* ? */

    if (configuring)
      (void) configure();
    else {
      (void) execute_move();
    }
    break;
  case 8: /* to = cur */
  case 9: /* to = cur */
    m->tocol = curcol;
    m->tolig = curlig ;

    if (configuring)
      (void) configure();
    else {
      (void) execute_move();
    }
    break;
  case 10: /* piece = cur piece ; prise = true */
    /* later : guess from position */
    m->piece = curpiece ;
    m->type = PRISE ;
    break;
  case 11: /* grand roque */
  case 12: /* petit roque */
    (void) execute_move();
    break;
  case 13: /* case of simpliest algebraic notation ;
	      only e2e4 : this is the transition from e2 to e4
	      also the case of move such as Nge2
	      from =cur; prise = FALSE;
	      also:
	      curcol = ...
	      */
    m->piece = curpiece ;
    m->fromcol = curcol ;
    m->fromlig = curlig;

    m->type = MOVE;
    curcol = lettertocol(c);
    break;
  case 14: /* promotion, the "=" */
  case 15: /* promotion, the piece name */
   m->tocol = curcol;
   m->tolig = curlig ;
   if (m->type == PRISE )
      m->type = PROM_ET_PRISE ;
    else
      m->type = PROMOTION ;
   if(num==14)
       m->promotion = curpiece = QUEEN ;
    else
       m->promotion = curpiece = piece(c) ;
    break;
  case 16: /* pawn captures in the form "cd" are now handled */
    m->type=PRISE;
    m->piece=PAWN;
    m->fromcol=curcol;
    m->tocol=lettertocol(c);
    break;
  case 17: /* execute move for promotion */
    (void) execute_move();
    break;
  case 18: /* dc5 - handle the 5 */
    m->tolig=lettertolig(c);
    break;
  case -1:
    break;
  default:
    break;
  }
  return(TRUE);
}

#ifdef __STDC__
int parse_number(char *token)
#else
int parse_number(token)
     char *token;
#endif
{
  int curmove = 0 ;
  int i;

  /* check coherency with internal numbering */
  i = 0;
  while (isdigit(token[i])) {
   curmove = curmove * 10 +  ((int) token[i++] - (int) '0' );
  }
  movecount = curmove ;
  if (checkmovenumbers)
    checkmovenumbers = check_numbering(m, movecount);
  return(TRUE);
}

#ifdef __STDC__
int parse_text(char *text)
#else
int parse_text(text)
     char *text;
#endif
{
  output_text(dr,T_TEXT, text, 0,0);
  return(TRUE);
}
#ifdef __STDC__
int parse_pgn_text(char *text)
#else
int parse_pgn_text(text)
     char *text;
#endif
{
  /* over ride null text driver */
  output_text(dr,T_TEXT, text, 0,TRUE);
  return(TRUE);
}

#ifdef __STDC__
int parse_comment(char *com)
#else
int parse_comment(com)
     char *com;
#endif
{
  int t;

  if (com[0] == '$')
    /* we look in the long ascii table */
    t = find_keyword(com_long, NUM_COM_CODE, NUM_COM_CODE, com, TRUE);
  else {
    /* we look for the comment in the short ascii table */
    t = find_keyword(com_short, NUM_COM_CODE, NUM_COM_CODE, com,FALSE);
    if (t == NUM_COM_CODE)
      fprintf (stderr,"\nUnrecognized comment: \"%s\"\n",com);
  }
  if (t != NUM_COM_CODE && strcmp(com,"+")!=0 && strcmp(com,"#")!=0)
    output_text(dr,T_COMMENT, com, t,0);
  return(TRUE);
}

/* printCheck - print Check symbol or Mate symbol
   Parameters:
   int mate - TRUE if mate; FALSE if Check
*/
#ifdef __STDC__
void printCheck(int mate)
#else
void printCheck()
    int mate;
#endif
{
 int t;
 char *sym;
 sym=mate?"#":"+";
 t=find_keyword(com_short,NUM_COM_CODE,NUM_COM_CODE,sym,FALSE);
 output_text(dr,T_COMMENT,sym,t,(dr->type==D_PGN));

}



/* Added by Ed Maillet */
/* Handles 0-1 0:1 1-0 1:0 1/2 1/2-1/2*/
#ifdef __STDC__
int parse_result(char *buf)
#else
int parse_result(buf)
     char *buf;
#endif
{
  if(dr->type!=D_PGN && dr->type!=D_CHESSARC)
     output_text(dr,T_SCORE, buf, 0,0);
  else
   { SetPGNString(PGN_RESULT,buf); }
   output_end_game(dr);
  return(TRUE);
}
#ifdef __STDC__
int parse_keyword(char *token, char *text)
#else
int parse_keyword(token,text)
     char *token;
     char *text;
#endif
{
  char c;

  switch (find_keyword(keywords, NBKEYWORD, KNULL, token, TRUE)) {
  case START:
    /* don't forget we are configuring the previous move */
    /* -> move 0, black */
    if(dr->type==D_ZARKOV)
       setup_zarkov_board(dr,tos,!m->whiteturn);
    if(dr->type==D_CM)
       setup_cm_board(dr,tos,!m->whiteturn);
    configuring = FALSE;
    break;
  case CLEAR:
    clear_board(tos);
    m= theplay->chain;
    free_move_list(m);
    break;
  case SHOWBOARD:
    output_board(dr,tos);
    break;
  case TOWHITE:
    /* don't forget we are configuring the previous move */
    /* reset to 0,black --> 1,white */
    m->move = 0;
    m->whiteturn = FALSE;
    break;
  case TOBLACK:
    /* reset to 1,white -> 1 black */
    m->move = 1;
    m->whiteturn = TRUE;
    dr->intrupt = TRUE;
    break;
  case CONFIGWH:
    configuring = TRUE ;
    configside = WHITE;
    m= theplay->chain;
    free_move_list(m);
    break;
  case CONFIGBL:
    configuring = TRUE ;
    configside = BLACK;
    m= theplay->chain;
    free_move_list(m);
    break;
  case DEFAULTP:
    init_board(tos);
    m= theplay->chain;
    free_move_list(m);
    break;
  case TITLE:
    output_text(dr, T_TITLE, text, NULL,0);
    break;
  case SUBTITLE:
    output_text(dr, T_SUBTITLE, text, NULL,0);
    break;
  case LANGUE:
    in_language = find_keyword (t_language, NBLANGUAGES, in_language,
			       text,TRUE);
    associe_traduction( &in_table, in_language);
    break;
  case SPECIAL: /* all input, up to \n is copied to output */
    while ((( c = getc(infile)) != EOF) && (c != '\n'))
      (void) putc (c,dr->outfile);
    putc ('\n', dr->outfile);
    break;
  case KNULL:
    break;
  default:
    fprintf(stderr,"unknown keyword %s\n",token);
    break;
  }
  return(TRUE);
}

#ifdef __STDC__
int parse_roque(char *token)
#else
int parse_roque(token)
     char * token;
#endif
{ 
  int i;

  for (i=0; i < NBROQUE && (strcmp(c_roque[i],token)!=0); i++) ;
  if ( i < NBROQUE ) {

    m = add_trailing_move(m);
    init_parse(m);

    if (strlen(token) == 3) {
      m->type = PETITROQUE ;
      (void) execute(12,DUMMYCHAR);
    } else {
      m->type = GRANDROQUE ;
      (void) execute(11,DUMMYCHAR);
    }
    /*(void) fprintf(stderr,"ROQUE\n");*/
    return(TRUE);
  }

  return(FALSE);
}

#ifdef __STDC__
int  parse_move(char *token)
#else
int  parse_move(token)
     char *token;
#endif
{
  register int i;
  int correcte = FALSE ;
  int erreursyntaxe = FALSE ;
  int etat =0;
  int code;

  if(m->type!=EMPTYNODE) /* add_variation has added this node already */
     m = add_trailing_move(m);
  init_parse(m);
  m->type = MOVE;

  i=0;
  while ( !correcte && !erreursyntaxe ) {
    code = typechar(token[i]);
    (void) execute(action[etat][code],token[i]);
    etat = transit[etat][code] ;
    if (etat == -1)
      erreursyntaxe = TRUE;
    if (etat == FINAL)
      correcte = TRUE ;
    i++;
  }
  if (erreursyntaxe) {
    message((stderr,"\n%s, line %d: I don't understand this move: \"%s\"",
	   filename, lineno , token));
    return(FALSE);
  }
  if (correcte) {
    /*(void) fprintf(stderr, "ia panimaiou, davai\n");*/
  }
  /*init_parse(m);*/
  dr->ingame=TRUE;
  return(TRUE);
}

#ifdef __STDC__
void init_parse(depl *m)
#else
void init_parse(m)
     depl * m ;
#endif
{

  /* global position and piece variable initialised to 0
     */
  /* move and whiteturn unchanged */

  m->type = MOVE ;

  curpiece = m->piece = VOID ;
  curcol = m->tocol = m->fromcol = 0;
  curlig = m->tolig = m->fromlig = 0;

  m->promotion = VOID;
  m->prise = VOID;
  m->is_check = FALSE ;
  m->ambfile=0;
  m->ambrank=0;
  curdigit = curmove = 0;

  /*if (movecount != m->move)
    (void) fprintf(stderr,"problem in move numbering: %d vs %d\n",
		   m->move, movecount);*/

}

/* ------------------- top routines -------------------- */

/* cette fonction analyse les arguments de la ligne de commande
   */
#ifdef __STDC__
int parse_options(int argc,char *argv[])
#else
int parse_options(argc,argv)
     int argc;
     char * argv[];
#endif
{
  int narg =1 ;
  int i;
  register int c;
  char cp[132];
  char chaine[MAXTOKLEN];

  infile = stdin;
  (void) sprintf (filename, "\"%s\"", "stdin" );

  nb_move_to_dsp = 0;

  while (narg < argc ) {
    (void) strcpy (cp,argv[narg]);
    switch (cp[0]) {
    case '-' :
      if(strcmp(&cp[1],"column")==0)
         { dr->column=TRUE;break; }
      switch (cp[1]) {
      case 'f' : /* from langage */
	if  ((narg+1) >= argc )
	  fatal((stderr,"missing argument to %s option",cp));
	narg++ ;
	in_language = find_keyword (t_language, NBLANGUAGES,
				    IN_LANG,
				    argv[narg],TRUE);
	break;
      case 't' : /* to langage */
	if  ((narg+1) >= argc )
	  fatal((stderr,"missing argument to %s option",cp));
	narg++ ;
	out_language = find_keyword (t_language, NBLANGUAGES,
				     OUT_LANG,
				     argv[narg],TRUE);
	break;
      case 'o' : /* next arg is output file */
	narg++ ;
	if ((dr->outfile = fopen (argv[narg],"w+")) == NULL) {
	  (void) fprintf (stderr,"can't open %s output file\n",argv[narg]);
	  (void) fprintf (stderr,"assume stdout for output\n");
	}
	break;
      case 'e':
	if  ((narg+1) >= argc )
	  fatal((stderr,"missing argument to %s option",cp));
	narg++ ;

	i=0;
	nb_move_to_dsp = 0;
	move_to_display[nb_move_to_dsp] = 0;
	while (isdigit(argv[narg][i])) {
	  move_to_display[nb_move_to_dsp] =
	    ((int) argv[narg][i] - (int) '0')
	      + move_to_display[nb_move_to_dsp] * 10;
	  i++;
	}
	nb_move_to_dsp++;
	stop_at_display = TRUE;
	break;
      case 'c':
	if  ((narg+1) >= argc )
	  fatal((stderr,"missing argument to %s option",cp));
	narg++ ;

	i=0;
	while (isdigit(argv[narg][i])) {
	  move_to_display[nb_move_to_dsp] = 0;
	  while (isdigit(argv[narg][i])) {
	    move_to_display[nb_move_to_dsp] =
	      ((int) argv[narg][i] - (int) '0')
	      + move_to_display[nb_move_to_dsp] * 10;
	    i++;
	  }
	  nb_move_to_dsp++;

	  if (nb_move_to_dsp > NB_MOVE_TO_DISP)
	    fatal((stderr,"max. number of move to display exceeded"));

	  /* process next number */
	  if (argv[narg][i] == ',')
	    i++;
	}
	break;
      case 'a': /* algebraic output */
	dr->output_move_format = ALGEBRAIC;
	break;
      case 's':  /* shortened output */
	dr->output_move_format = SHORTENED;
	break;
      case 'b': /* display only the board, no move */
	dr->only_board = TRUE;
	break;
      case 'd': /* output driver */
	if  ((narg+1) >= argc )
	  fatal((stderr,"missing argument to %s option",cp));
	narg++ ;
	driver = find_keyword(t_output, NB_DRIVER, DEFAULT_DRIVER,
			      argv[narg],TRUE);
	break;
      case 'i': /* no headers */
	dr->print_headers = FALSE;
	break;
      case 'v': /* print version */
	/* this already done, so exit() */
	exit(0);
	break;
      case 'n': /* Don't print final board */
        dr->final_board=FALSE;
        break;
      case 'h': /* help file */
	(void) strcpy(chaine,LIB_DIR);
        if ((fhelp = fopen(strcat(chaine,HELP_FILE),"r")) == NULL)
          fatal((stderr,"Can't find help file.\n"));
        else {
          while ((c = getc(fhelp)) != EOF)
            (void) fputc(c,stderr);
          (void) fclose(fhelp);
	  exit(0);
        }
         break;
      default:
	error((stderr,"\nUnknown command line options %s\n", cp));
	break;
      }
      break;
    default: /* assume this is the input file */
      if ((infile = fopen (cp,"r")) == NULL)
	fatal((stderr,"can't open %s input file\n",cp));
      (void) sprintf (filename, "\"%s\"", cp );
    }
    narg++;
  } /* process next arg */
  return(argc);
}

#ifdef __STDC__
void close_files(void)
#else
void close_files()
#endif
{
  if (!((infile == stdin)||(infile == NULL)))
    (void) fclose(infile);
  if (dr->outfile != stdout )
    (void) fclose(dr->outfile);
}

#ifdef __STDC__
int associe_traduction (char **table, int language)
#else
int associe_traduction (table, language)
char ** table;
int language ;
#endif
{
  if (language < 0 || (language >= NBLANGUAGES))
    error((stderr,"\nUnknown language\n"));
  else
    *table = c_language[language];
  return(language);
}

#ifdef __STDC__
void print_all_play(play *p)
#else
void print_all_play(p)
play *p;
#endif
{
  depl *d;
  d = p->chain;
  while (d->next != NULL)
  {
    d = d->next;
    output_move(dr,d);
  }
}

/* ------------- main --------------------- */

#ifdef __STDC__
int main(int argc,char *argv[])
#else
int main(argc,argv)
     int argc;
     char * argv[];
#endif
{
  (void) fprintf(stderr,"%s\n",version_string);

  /* allocation of driver descriptor */
  dr = new_driver();

  /* default configuration */
  init_driver(dr,DEFAULT_DRIVER);
  (void) associe_traduction(&in_table,  IN_LANG );
  (void) associe_traduction(&(dr->out_table), OUT_LANG);

  (void) parse_options(argc,argv);

  (void) associe_traduction (&in_table, in_language);
  (void) associe_traduction (&(dr->out_table), out_language);

  /* assoc driver */
  init_driver(dr,driver);

  configuring = FALSE;
  configside = VOID;

  /* initialise output file */
  output_init(dr);

  if (error_flag)
    fatal((stderr,"\nToo many errors"));

  /* allocation of board descriptor */
  tos = new_board();
  init_board(tos);

  /* allocation of move descriptor */
  m = new_move();
  m->type = HEADNODE;
  /*init_move(m);*/

  /* allocation of the play descriptor */
  theplay = (play *) malloc (sizeof(play)) ;
  theplay->initial = tos ;
  theplay->chain   = m ;
  movecount = 1;

  /* main analysis routine */
  yyin = infile ;
  yyout = stderr ;

  /*init_parse(m); */
  yylex();

  if (error_flag) {
    error((stderr,"\nLast valid position:\n"));
    output_board(dr,tos);
    fatal((stderr,"\nToo many errors"));
  }

  /* terminates output files */
  output_end(dr);

/* close files */
  close_files();

#ifdef __STDC__
#else
  /* exit properly */
  if (error_flag == FALSE) /* exit without error */
    exit(0);
  else /* exit with error */
    exit(1);
#endif
}
