/*
  ChessPrn program by Ed Maillet
  @(#)notation.h 1.0 Dated 3/93
  Copyright (C) 1993  Ed Maillet
  Modified and based on Notation 4.7 by Henry Thomas
 */
#ifndef _NOTATION_HEADERS
#define _NOTATION_HEADERS


/* ---- KEYWORDS ----  */

/* prefix for keywords */
#define PREFIX '@'


#define START     0
#define CLEAR     1
#define SHOWBOARD 2
#define TOWHITE   3
#define TOBLACK   4
#define CONFIGWH  5
#define CONFIGBL  6
#define DEFAULTP  7
#define TITLE	  8
#define SUBTITLE  9
#define LANGUE    10
#define SPECIAL   11
#define KNULL     12

#define NBKEYWORD (KNULL+1)
/* PGN Strings */
#define PGN_EVENT   0
#define PGN_SITE    1
#define PGN_DATE    2
#define PGN_ROUND   3
#define PGN_WHITE   4
#define PGN_BLACK   5
#define PGN_RESULT  6
#define PGN_COMMENT 7

/* ---- LANGUAGES ---- */

#define FRENCH 		0
#define ENGLISH 	1
#define ITALIAN 	2
#define SPANISH 	3
#define GERMAN  	4
#define DUTCH   	5
#define CZECH		6
#define HUNGARIAN      	7
#define POLISH		8
#define ROMANIAN       	9
#define FIDE		10
#define DANISH          11
#define ESTONIAN        12
#define FINNISH         13
#define NORWEGIAN       14
#define PORTUGUESE      15
#define SWEDISH         16

#define USERDEF		17

#define NBLANGUAGES 	(USERDEF+0)

#define OVERRIDE 777 /* over ride null text driver with ascii */

/* a convenient way  to get a value for NUM_COMMENT
   look at GNU-CC source, it's full of *neat* tricks
   */

#define CHESSSYMB(LET,LASC,SASC,TEX,PS,ENG,FRA) LET,

enum com_code {
#include "chesssym.def"
  LAST_UNUSED_NUM_COM_CODE
  };
#undef CHESSSYMB

#define NUM_COM_CODE ((int) LAST_UNUSED_NUM_COM_CODE)

/* ---- MISC TABLES ---- */

/* roque ascii table */
#define NBROQUE     6
#define SPETITROQUE 0
#define SGRANDROQUE 1


/* ---- DRIVERS ---- */

/* output drivers name table */
#define DEFAULT_DRIVER D_ASCII

/* ---- IO ---- */

extern FILE * infile;

/* ------------- service routines -------------------*/

/* Output an error message and exit */
#define fatal(A) (void) fprintf A , close_files() , exit(1)

/* Output an error message and set the error flag */
#define error(A) (void) fprintf A , error_flag = TRUE , (void)fflush(stderr)

/* Output an error message and set the error flag */
#define message(A) (void) fprintf A 

#define MALLOC(T)  (T *)malloc((unsigned)sizeof(T))
#define ALLOCP(P)  if ( (P) == NULL ) { fatal((stderr,"malloc failed")) ; };

/* ------- conversion of input character --------------------- */

#define lettertocol(c)  ((c)-'a'+1)
#define lettertolig(c)  ((c)-'1'+1)
#define coltoletter(n)  ((n)-1+'a')
#define ligtoletter(n)  ((n)-1+'1')


/* ------- variables and functions --------------------------- */
extern play * theplay;
extern game * tos;
extern depl *m;


/* the stack itself */
extern stack_elt stack[];

extern char *version_string;
extern int configuring;
extern int configside;
extern char *in_table;
extern char *out_table;
extern char *c_roque[];
extern char *c_en_passant[];
extern char c_prise;
extern char *c_comments[];
extern int error_flag;

extern char *com_short[];
extern char *com_long[];


/* ------ parse and analysis routines ------ */

#ifdef __STDC__
extern int clear_pos(int lig, int col);
extern int configure(void);
extern void enter_variation(void);
extern void exit_variation(void);
extern int execute_move(void);
extern int typechar(char c);
extern int execute(int num, char c);
extern int parse_number(char *token);
extern int parse_text(char *text);
extern int parse_pgn_text(char *text);
extern int parse_comment(char *com);
extern int parse_result(char *com);
extern int parse_keyword(char *token, char *text);
extern int parse_roque(char *token);
extern int parse_move(char *token);
extern void init_parse(depl *m);
extern int parse_options(int argc, char *argv[]);
extern void close_files(void);
extern int associe_traduction(char **table, int language);
extern void print_all_play(play *p);
extern char GetLangChar(int lang,int piece);
extern void printCheck(int mate);

#else

extern int clear_pos(/*int lig, int col*/);
extern int configure(/*void*/);
extern void enter_variation(/*void*/);
extern void exit_variation(/*void*/);
extern int execute_move(/*void*/);
extern int typechar(/*char c*/);
extern int execute(/*int num, char c*/);
extern int parse_number(/*char *token*/);
extern int parse_text(/*char *text*/);
extern int parse_pgn_text(/*char *text*/);
extern int parse_comment(/*char *com*/);
extern int parse_result(/*char *com*/);
extern int parse_keyword(/*char *token, char *text*/);
extern int parse_roque(/*char *token*/);
extern int parse_move(/*char *token*/);
extern void init_parse(/*depl *m*/);
extern int parse_options(/*int argc, char *argv[]*/);
extern void close_files(/*void*/);
extern int associe_traduction(/*char **table, int language*/);
extern void print_all_play(/*play *p*/);
extern char GetLangChar(/*int lang,int piece*/);
extern void printCheck(/*int mate*/);

#endif

/* ------ top-level interface functions -------- */

#ifdef __STDC__
extern int parse_options(int argc, char *argv[]);
extern void close_files(void);
extern int associe_traduction(char **table, int langage);

#else
extern int parse_options(/*int argc, char *argv[]*/);
extern void close_files(/*void*/);
extern int associe_traduction(/*char **table, int langage*/);

#endif


#endif
